package net.sf.doolin.gui.app;

import net.sf.doolin.context.AbstractApplication;
import net.sf.doolin.context.ApplicationLauncher;
import net.sf.doolin.gui.app.support.ApplicationActionContext;
import net.sf.doolin.gui.service.DesktopService;
import net.sf.doolin.gui.service.GUIPreferences;
import net.sf.doolin.gui.service.InfoService;
import net.sf.doolin.gui.service.Parameters;
import net.sf.doolin.gui.service.support.DefaultGUIPreferences;
import net.sf.doolin.gui.service.support.DefaultInfoService;
import net.sf.doolin.gui.service.support.DefaultParameters;
import net.sf.doolin.gui.service.support.JDK6DesktopService;
import net.sf.doolin.util.Utils;
import net.sf.jstring.JStrings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.PostConstruct;
import javax.swing.*;
import java.util.*;

/**
 * GUI application manager.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class GUIApplication extends AbstractApplication {

	private static final Logger log = LoggerFactory
			.getLogger(GUIApplication.class);

	/**
	 * Stores the ID class, as defined at startup by the main class.
	 */
	private static Class<?> idClass;

	/**
	 * Returns the class that identifies the application
	 * 
	 * @return Identifying class
	 */
	public synchronized static Class<?> getIdClass() {
		if (idClass != null) {
			return idClass;
		} else {
			// ID class
			Throwable th = new Throwable();
			StackTraceElement[] stackTraces = th.getStackTrace();
			// Gets the last element in the stack trace
			StackTraceElement stackTraceElement = stackTraces[stackTraces.length - 1];
			String idClassName = stackTraceElement.getClassName();
			// OK
			log.info("ID class is " + idClassName);
			idClass = Utils.forClass(idClassName);
			return idClass;
		}
	}

	/**
	 * Launches a {@link GUIApplication} on the Swing EDT thread.
	 * 
	 * @param paths
	 *            Paths for the application configuration
	 * @param args
	 *            Arguments for the application
	 */
	public static void launchWithArgs(final String[] paths, final String[] args) {
		// Computes the ID class before going in EDT
		getIdClass();
		// Launches the application in EDT
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				ApplicationLauncher.launchWithArgs(paths, args);
			}
		});
	}

	private GUIStartup startup;

	private Map<Class<?>, Object> services = new HashMap<Class<?>, Object>();

	private final ApplicationActionContext actionContext;

	/**
	 * List of opened applications
	 */
	private static Set<GUIApplication> applications = Collections
			.synchronizedSet(new HashSet<GUIApplication>());

	/**
	 * Constructor
	 */
	public GUIApplication() {
		this.actionContext = new ApplicationActionContext(this);
	}

	/**
	 * Exits the application
	 * 
	 * @param code
	 *            Exit code
	 */
	public synchronized void exit(int code) {
		// Removes the application from the list of opened applications
		applications.remove(this);
		// Checks if the application is alone in the JVM
		if (applications.isEmpty()) {
			System.exit(code);
		}
	}

	/**
	 * If the bean is not defined in the Spring configuration, searches in the
	 * predefined services.
	 */
	@SuppressWarnings("unchecked")
	@Override
	public <T> T getBean(Class<T> idClass) {
		if (isDefined(idClass)) {
			return super.getBean(idClass);
		} else {
			Object o = this.services.get(idClass);
			if (o != null) {
				return (T) o;
			} else {
				throw new RuntimeException(String.format(
						"No service is defined for %s", idClass));
			}
		}
	}

	/**
	 * Gets the information service for the application
	 * 
	 * @return Information service
	 */
	public InfoService getInfoService() {
		return getBean(InfoService.class);
	}

	/**
	 * Gets the parameter service for this application.
	 * 
	 * @return Parameter service
	 */
	public Parameters getParameters() {
		return getBean(Parameters.class);
	}

	/**
	 * Gets the preferences for the application
	 * 
	 * @return Preferences
	 */
	public GUIPreferences getPreferences() {
		return getBean(GUIPreferences.class);
	}

	/**
	 * Returns the start-up policy
	 * 
	 * @return Startup policy
	 */
	public GUIStartup getStartup() {
		return this.startup;
	}

	/**
	 * Initialises the defaults services.
	 */
	@PostConstruct
	public void init() {
		// Bundles
        JStrings.add("net.sf.doolin.util.DefaultStrings");
		JStrings.add("net.sf.doolin.gui.GUIBundle");
		JStrings.add("net.sf.doolin.gui.GUIErrorMessages");
		// Registers default services
		initDefaultServices();
	}

	/**
	 * Initialises the defaults services.
	 */
	protected void initDefaultServices() {
		// Default preferences
		registerService(GUIPreferences.class, new DefaultGUIPreferences());
		// Default desktop service
		registerService(DesktopService.class, new JDK6DesktopService());
		// Parameters
		registerService(Parameters.class, new DefaultParameters());
		// Logger
		registerService(InfoService.class, new DefaultInfoService());
	}

	/**
	 * Registers a default service for this application
	 * 
	 * @param <T>
	 *            Type of service
	 * @param serviceClass
	 *            Service interface
	 * @param service
	 *            Service implementation
	 */
	protected <T> void registerService(Class<T> serviceClass, T service) {
		log.debug(String.format("Using %2$s as implementation for %1$s",
				serviceClass.getName(), service.getClass().getName()));
		this.services.put(serviceClass, service);
	}

	/**
	 * Sets the start-up policy for this application.
	 * 
	 * @param startup
	 *            Startup policy
	 */
	public void setStartup(GUIStartup startup) {
		this.startup = startup;
	}

	/**
	 * Delegates to the start-up policy.
	 * 
	 * @see GUIStartup#start(GUIApplication, String[])
	 */
	@Override
	public void start(final String[] arguments) {
		applications.add(this);
		if (this.startup != null) {
			SwingUtilities.invokeLater(new Runnable() {

				@Override
				public void run() {
					try {
						GUIApplication.this.startup.start(GUIApplication.this,
								arguments);
					} catch (Exception th) {
						getInfoService().error(
								GUIApplication.this.actionContext, th,
								"GUIApplication.startup.error");
					}
				}

			});
		} else {
			log.warn("No startup policy has been defined.");
		}
	}
}
