package net.sf.doolin.gui.edit;

import net.sf.doolin.bus.Bus;
import net.sf.doolin.bus.support.AbstractClassSubscriber;
import net.sf.doolin.bus.support.Value;
import net.sf.doolin.gui.action.AbstractGUIAction;
import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.display.AbstractDisplayStateHandler;
import net.sf.doolin.gui.display.DisplayState;
import net.sf.doolin.gui.display.DisplayStateHandler;
import net.sf.doolin.gui.expression.GUIExpression;
import net.sf.doolin.gui.util.GUIStrings;
import net.sf.doolin.util.Utils;

import org.springframework.beans.factory.BeanNameAware;

/**
 * This {@link GUIAction} is a support for the tasks that can be done and
 * undone.
 * 
 * @author Damien Coraboeuf
 * 
 */
public abstract class GUIEditAction extends AbstractGUIAction implements
		BeanNameAware {

	private String editManagerPropertyPath;
	private String id;

	private final Value<String> textValue = new Value<String>();

	/**
	 * Constructor.
	 * 
	 * It sets a default {@link DisplayStateHandler} that enables or disables
	 * this action according to the state of the task manager.
	 */
	public GUIEditAction() {
		setDisplayStateHandler(new AbstractDisplayStateHandler() {

			@Override
			public DisplayState getDisplayState(ActionContext actionContext) {
				// Gets the edit manager
				GUIEditManager editManager = getEditManager(actionContext);
				// OK
				return getActionState(editManager);
			}

			@Override
			public void subscribe(ActionContext actionContext,
					final Runnable task) {
				// Gets the edit manager
				final GUIEditManager editManager = getEditManager(actionContext);
				// Subscription
				Bus
						.get()
						.subscribe(
								new AbstractClassSubscriber<GUIEditManagerUpdateMessage>(
										GUIEditManagerUpdateMessage.class,
										actionContext.getSubscriberValidator()) {

									@Override
									public boolean acceptMessage(
											GUIEditManagerUpdateMessage message) {
										return message.getEditManager() == editManager;
									}

									@Override
									public String getExecutionDescription() {
										return String
												.format(
														"Runs tasks for GUIEditAction %s",
														GUIEditAction.this.id);
									}

									@Override
									public void receive(
											GUIEditManagerUpdateMessage message) {
										task.run();
									}
								});
			}
		});
	}

	@Override
	protected void doExecute(ActionContext context) throws Exception {
		// Gets the edit manager
		GUIEditManager editManager = getEditManager(context);
		// Performs the task
		execute(editManager);
	}

	/**
	 * Executes this action for its associated {@link GUIEditManager}.
	 * 
	 * @param editManager
	 *            Task manager
	 */
	protected abstract void execute(GUIEditManager editManager);

	/**
	 * Gets the display state of this action according to the state of the task
	 * manager
	 * 
	 * @param editManager
	 *            task manager
	 * @return Display state
	 */
	protected abstract DisplayState getActionState(GUIEditManager editManager);

	/**
	 * Gets the text of this action according to the state of the task manager
	 * 
	 * @param editManager
	 *            Task manager
	 * @return Display text
	 */
	protected abstract String getActionText(GUIEditManager editManager);

	/**
	 * Gets the task manager from the action context
	 * 
	 * @param actionContext
	 *            Action context
	 * @return Task manager
	 * @see ActionContext#getData()
	 * @see #setEditManagerPropertyPath(String)
	 */
	protected GUIEditManager getEditManager(ActionContext actionContext) {
		return (GUIEditManager) Utils.getProperty(actionContext.getData(),
				this.editManagerPropertyPath);
	}

	/**
	 * Gets the path to the property that holds the task manager.
	 * 
	 * @return Property path
	 */
	public String getEditManagerPropertyPath() {
		return this.editManagerPropertyPath;
	}

	@Override
	public GUIExpression getIconIDExpression(ActionContext actionContext) {
		String key = getId() + ".icon";
		if (GUIStrings.isDefined(key)) {
			String value = GUIStrings.get(key);
			GUIExpression expression = new GUIExpression(actionContext
					.getSubscriberValidator(), actionContext.getData(), value);
			return expression;
		} else {
			return null;
		}
	}

	@Override
	public String getId() {
		return this.id;
	}

	@Override
	public GUIExpression getShortcutExpression(ActionContext actionContext) {
		// Shortcut id
		String key = getId() + ".key";
		if (GUIStrings.isDefined(key)) {
			String value = GUIStrings.get(key);
			GUIExpression expression = new GUIExpression(actionContext
					.getSubscriberValidator(), actionContext.getData(), value);
			return expression;
		} else {
			return null;
		}
	}

	@Override
	public GUIExpression getTextExpression(ActionContext actionContext) {
		// Edition manager
		final GUIEditManager editManager = getEditManager(actionContext);
		// Initial value
		String text = getActionText(editManager);
		this.textValue.setValue(text);
		// Listens for changes
		Bus.get().subscribe(
				new AbstractClassSubscriber<GUIEditManagerUpdateMessage>(
						GUIEditManagerUpdateMessage.class, actionContext
								.getSubscriberValidator()) {

					@Override
					public boolean acceptMessage(
							GUIEditManagerUpdateMessage message) {
						return message.getEditManager() == editManager;
					}

					@Override
					public String getExecutionDescription() {
						return String.format(
								"Sets the text of the GUIEditAction %s",
								GUIEditAction.this.id);
					}

					@Override
					public void receive(GUIEditManagerUpdateMessage message) {
						String text = getActionText(editManager);
						GUIEditAction.this.textValue.setValue(text);
					}
				});
		// Text expression
		return new GUIExpression(actionContext.getSubscriberValidator(),
				this.textValue, "@{value}");
	}

	@Override
	public GUIExpression getTipExpression(ActionContext actionContext) {
		return getTextExpression(actionContext);
	}

	@Override
	public void setBeanName(String name) {
		this.id = name;
	}

	/**
	 * Sets the path to the property that holds the task manager.
	 * 
	 * @param undoManagerPropertyPath
	 *            Property path
	 */
	public void setEditManagerPropertyPath(String undoManagerPropertyPath) {
		this.editManagerPropertyPath = undoManagerPropertyPath;
	}

	/**
	 * Sets the ID of this action
	 * 
	 * @param id
	 *            ID
	 */
	public void setId(String id) {
		this.id = id;
	}

}
