package net.sf.doolin.gui.field;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.util.Adapter;
import net.sf.doolin.util.Utils;

/**
 * Abstract field descriptor for fields based on a property.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 */
public abstract class AbstractPropertyFieldDescriptor<V> extends
		AbstractFieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private String propertyPath;

	private Adapter<Object, Object> adapter;

	/**
	 * Gets the property path.
	 * 
	 * @return the property path
	 */
	public String getPropertyPath() {
		return this.propertyPath;
	}

	/**
	 * Path to the property linked to the field
	 * 
	 * @param propertyPath
	 *            the new property path
	 */
	public void setPropertyPath(String propertyPath) {
		this.propertyPath = propertyPath;
	}

	/**
	 * Returns the adapter used between the model and the field
	 * 
	 * @return Adapter
	 */
	public Adapter<Object, Object> getAdapter() {
		return this.adapter;
	}

	/**
	 * Sets the adapter used between the model and the field
	 * 
	 * @param adapter
	 *            Adapter
	 */
	public void setAdapter(Adapter<Object, Object> adapter) {
		this.adapter = adapter;
	}

	/**
	 * Updates the view model according to the property set by this field. Any
	 * defined {@link #setAdapter(Adapter) adapter} is used before setting the
	 * property.
	 * 
	 * @param view
	 *            Hosting view
	 * @param value
	 *            New value for the property.
	 * @see GUIView#getViewData()
	 */
	protected void setProperty(GUIView<V> view, Object value) {
		if (this.adapter != null) {
			value = this.adapter.convertTargetToSubject(value);
		}
		Utils.setProperty(view.getViewData(), this.propertyPath, value);
	}

	/**
	 * Updates a target property each time the field property is updated.
	 * 
	 * @param view
	 *            Hosting view
	 * @param target
	 *            Target bean
	 * @param targetProperty
	 *            Path to the target property
	 */
	protected void subscribe(final GUIView<V> view, final Object target,
			final String targetProperty) {
		Runnable task = new Runnable() {
			@Override
			public void run() {
				Object propertyValue = getProperty(view);
				Utils.setProperty(target, targetProperty, propertyValue);
			}
		};
		PropertyChangeSupport.subscribe(view.getActionContext()
				.getSubscriberValidator(), view.getViewData(),
				this.propertyPath, task);
		task.run();
	}

	/**
	 * Runs a task each time the field property is updated.
	 * 
	 * @param view
	 *            Hosting view
	 * @param runnable
	 *            Task to run each time the field property is updated
	 */
	protected void subscribe(final GUIView<V> view, Runnable runnable) {
		PropertyChangeSupport.subscribe(view.getActionContext()
				.getSubscriberValidator(), view.getViewData(),
				this.propertyPath, runnable);
	}

	/**
	 * Gets the field property from the view.
	 * 
	 * @param view
	 *            Hosting view
	 * @return Field property
	 */
	protected Object getProperty(GUIView<V> view) {
		Object value = Utils.getProperty(view.getViewData(), this.propertyPath);
		if (this.adapter != null) {
			value = this.adapter.convertSubjectToTarget(value);
		}
		return value;
	}

}
