package net.sf.doolin.gui.field;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JRootPane;
import javax.swing.SwingUtilities;

import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.display.AndDisplayStateHandler;
import net.sf.doolin.gui.display.DisplayStateHandler;
import net.sf.doolin.gui.expression.GUIExpression;
import net.sf.doolin.gui.field.support.SimpleField;
import net.sf.doolin.gui.service.IconSize;
import net.sf.doolin.gui.view.GUIView;

import org.springframework.beans.factory.annotation.Required;

/**
 * {@link JButton} field descriptor.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 */
public class FieldButton<V> extends AbstractFieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private GUIAction action;

	private boolean defaultButton;

	private String textExpression;

	private IconSize iconSize;

	/**
	 * Creates the field.
	 * 
	 * @param view
	 *            the view
	 * 
	 * @return the field< v>
	 * 
	 * @see net.sf.doolin.gui.action.swing.ActionFactory#createAction(ActionContext,
	 *      GUIAction, IconSize)
	 */
	@Override
	public Field<V> createField(GUIView<V> view) {
		// Creates the button
		final JButton button = new JButton();
		// Field
		SimpleField<V> field = new SimpleField<V>(view, this, button);
		// Creates an action context for the button
		ActionContext actionContext = field.getActionContext();
		// Icon size
		IconSize iconSize = getActualIconSize(view);
		// Creates the action
		Action swingAction = getActionFactory().createAction(actionContext,
				this.action, iconSize);
		button.setAction(swingAction);
		// Alternate text
		if (this.textExpression != null) {
			// Expression
			final GUIExpression expression = new GUIExpression(actionContext
					.getSubscriberValidator(), actionContext.getData(),
					this.textExpression);
			// Connection
			expression.connect(actionContext.getSubscriberValidator(), button,
					"text");
		}
		// OK
		return field;
	}

	/**
	 * Gets the action.
	 * 
	 * @return the action
	 */
	public GUIAction getAction() {
		return this.action;
	}

	protected IconSize getActualIconSize(GUIView<V> view) {
		if (this.iconSize != null) {
			return this.iconSize;
		} else {
			return view.getApplication().getParameters().getEnum(
					"FieldButton.IconSize", IconSize.class, IconSize.SMALL);
		}
	}

	/**
	 * Uses a conjunction between the action's handler and the button's handler.
	 * 
	 * {@inheritDoc}
	 * 
	 * @see net.sf.doolin.gui.display.AndDisplayStateHandler
	 */
	@Override
	public DisplayStateHandler getDisplayStateHandler() {
		DisplayStateHandler actionHandler = this.action
				.getDisplayStateHandler();
		DisplayStateHandler buttonHandler = super.getDisplayStateHandler();
		return new AndDisplayStateHandler(buttonHandler, actionHandler);
	}

	public IconSize getIconSize() {
		return this.iconSize;
	}

	/**
	 * Returns the expression for an alternative text
	 * 
	 * @return Expression
	 */
	public String getTextExpression() {
		return this.textExpression;
	}

	/**
	 * Checks if is default button.
	 * 
	 * @return true, if is default button
	 */
	public boolean isDefaultButton() {
		return this.defaultButton;
	}

	@Override
	public void onShow(Field<V> field) {
		super.onShow(field);
		JComponent fieldComponent = field.getFieldComponent();
		if (this.defaultButton && fieldComponent instanceof JButton) {
			JRootPane rootPane = SwingUtilities.getRootPane(fieldComponent);
			if (rootPane != null) {
				rootPane.setDefaultButton((JButton) fieldComponent);
			}
		}
	}

	/**
	 * Associated action to execute
	 * 
	 * @param action
	 *            the new action
	 */
	@Required
	public void setAction(GUIAction action) {
		this.action = action;
	}

	/**
	 * Sets to <code>true</code> if this is the default button.
	 * 
	 * @param defaultButton
	 *            <code>true</code> if this is the default button
	 */
	public void setDefaultButton(boolean defaultButton) {
		this.defaultButton = defaultButton;
	}

	public void setIconSize(IconSize iconSize) {
		this.iconSize = iconSize;
	}

	/**
	 * Sets the the expression for an alternative text
	 * 
	 * @param textExpression
	 *            Expression
	 */
	public void setTextExpression(String textExpression) {
		notify("textExpression", this.textExpression,
				this.textExpression = textExpression);
	}

}
