package net.sf.doolin.gui.field;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.ParseException;

import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.SwingConstants;
import javax.swing.JFormattedTextField.AbstractFormatter;

import net.sf.doolin.gui.field.support.SimpleField;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.util.Utils;

import org.apache.commons.lang.StringUtils;

/**
 * Field for an {@link BigDecimal} value.
 * <p>
 * It uses a {@link JFormattedTextField} component.
 * <p>
 * The bounds of the component are defined, by order or priority:
 * <ol>
 * <li>property path - either {@link #setMinimumProperty(String)} or
 * {@link #setMaximumProperty(String)}
 * <li>direct value - either {@link #setMinimum(BigDecimal)} or
 * {@link #setMaximum(BigDecimal)}
 * <li><code>null</code> (unbounded)
 * </ol>
 * 
 * @param <V>
 *            Type of object for the view
 */
public class FieldDecimal<V> extends AbstractPropertyFieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private BigDecimal minimum;
	private BigDecimal maximum;

	private String minimumProperty;
	private String maximumProperty;

	private boolean strictMin;
	private boolean strictMax;

	private int fractionDigits = 2;
	private int integerDigits = 10;
	private boolean fractionShowAll = true;

	protected BigDecimal computeMaximum(GUIView<V> view) {
		V data = view.getViewData();
		BigDecimal max;
		if (StringUtils.isNotBlank(this.maximumProperty)) {
			max = (BigDecimal) Utils.getProperty(data, this.maximumProperty);
		} else {
			max = this.maximum;
		}
		return max;
	}

	protected BigDecimal computeMinimum(GUIView<V> view) {
		V data = view.getViewData();
		BigDecimal min;
		if (StringUtils.isNotBlank(this.minimumProperty)) {
			min = (BigDecimal) Utils.getProperty(data, this.minimumProperty);
		} else {
			min = this.minimum;
		}
		return min;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Field<V> createField(final GUIView<V> view) {

		// Format to be used
		DecimalFormat decimalFormat = (DecimalFormat) DecimalFormat
				.getInstance();
		// Fraction digits
		decimalFormat.setMaximumFractionDigits(this.fractionDigits);
		if (this.fractionShowAll) {
			decimalFormat.setMinimumFractionDigits(this.fractionDigits);
		}
		// Integer digits
		decimalFormat.setMaximumIntegerDigits(this.integerDigits);

		// Field
		final JFormattedTextField field = new JFormattedTextField(decimalFormat);
		field.setHorizontalAlignment(SwingConstants.TRAILING);
		field.setColumns(this.integerDigits + 1 + this.fractionDigits);

		// Minimum and maximum
		field.setInputVerifier(new InputVerifier() {

			@Override
			public boolean verify(JComponent input) {
				// Gets the value
				BigDecimal value;
				try {
					value = getValue(field, false);
				} catch (NumberFormatException ex) {
					return false;
				}
				if (value == null) {
					return true;
				}
				// Gets the limits
				BigDecimal min = computeMinimum(view);
				BigDecimal max = computeMaximum(view);
				// Min
				if (min != null) {
					if (FieldDecimal.this.strictMin) {
						return value.compareTo(min) > 0;
					} else {
						return value.compareTo(min) >= 0;
					}
				}
				// Max
				if (max != null) {
					if (FieldDecimal.this.strictMax) {
						return value.compareTo(max) < 0;
					} else {
						return value.compareTo(max) <= 0;
					}
				}
				// OK
				return true;
			}
		});

		// Model -> Field
		Runnable updateField = new Runnable() {

			@Override
			public void run() {
				field.setValue(getProperty(view));
			}
		};
		subscribe(view, updateField);

		// Initialisation
		updateField.run();

		// Field -> Model
		field.addPropertyChangeListener("value", new PropertyChangeListener() {

			@Override
			public void propertyChange(PropertyChangeEvent evt) {
				BigDecimal bd = getValue(field, true);
				setProperty(view, bd);
			}
		});

		// OK
		return new SimpleField<V>(view, this, field);
	}

	public int getFractionDigits() {
		return this.fractionDigits;
	}

	public int getIntegerDigits() {
		return this.integerDigits;
	}

	/**
	 * Gets the maximum.
	 * 
	 * @return the maximum
	 */
	public BigDecimal getMaximum() {
		return this.maximum;
	}

	/**
	 * Returns the path to the property that contains the maximum value for this
	 * field.
	 * 
	 * @return Property path
	 */
	public String getMaximumProperty() {
		return this.maximumProperty;
	}

	/**
	 * Gets the minimum.
	 * 
	 * @return the minimum
	 */
	public BigDecimal getMinimum() {
		return this.minimum;
	}

	/**
	 * Returns the path to the property that contains the minimum value for this
	 * field.
	 * 
	 * @return Property path
	 */
	public String getMinimumProperty() {
		return this.minimumProperty;
	}

	protected BigDecimal getValue(JFormattedTextField field,
			boolean useFieldValue) {
		Number value;
		if (useFieldValue) {
			value = (Number) field.getValue();
		} else {
			AbstractFormatter formatter = field.getFormatter();
			String text = field.getText();
			try {
				value = (Number) formatter.stringToValue(text);
			} catch (ParseException e) {
				throw new NumberFormatException(text);
			}
		}
		BigDecimal bd;
		if (value instanceof BigDecimal) {
			bd = (BigDecimal) value;
		} else {
			bd = new BigDecimal(value.toString());
		}
		return bd;
	}

	public boolean isFractionShowAll() {
		return this.fractionShowAll;
	}

	public boolean isStrictMax() {
		return this.strictMax;
	}

	public boolean isStrictMin() {
		return this.strictMin;
	}

	public void setFractionDigits(int fractionDigits) {
		this.fractionDigits = fractionDigits;
	}

	public void setFractionShowAll(boolean fractionShowAll) {
		this.fractionShowAll = fractionShowAll;
	}

	public void setIntegerDigits(int integerDigits) {
		this.integerDigits = integerDigits;
	}

	/**
	 * Sets the maximum.
	 * 
	 * @param maximum
	 *            the new maximum
	 */
	public void setMaximum(BigDecimal maximum) {
		this.maximum = maximum;
	}

	/**
	 * Sets the path to the property that contains the maximum value for this
	 * field.
	 * 
	 * @param maximumProperty
	 *            Property path
	 */
	public void setMaximumProperty(String maximumProperty) {
		this.maximumProperty = maximumProperty;
	}

	/**
	 * Sets the minimum.
	 * 
	 * @param minimum
	 *            the new minimum
	 */
	public void setMinimum(BigDecimal minimum) {
		this.minimum = minimum;
	}

	/**
	 * Sets the path to the property that contains the minimum value for this
	 * field.
	 * 
	 * @param minimumProperty
	 *            Property path
	 */
	public void setMinimumProperty(String minimumProperty) {
		this.minimumProperty = minimumProperty;
	}

	public void setStrictMax(boolean strictMax) {
		this.strictMax = strictMax;
	}

	public void setStrictMin(boolean strictMin) {
		this.strictMin = strictMin;
	}

}
