package net.sf.doolin.gui.field.list;

import javax.swing.DefaultListCellRenderer;
import javax.swing.ListCellRenderer;

import net.sf.doolin.gui.field.AbstractPropertyFieldDescriptor;
import net.sf.doolin.gui.swing.DefaultLabelInfoProvider;
import net.sf.doolin.gui.swing.LabelInfoListCellRenderer;
import net.sf.doolin.gui.swing.LabelInfoProvider;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.util.factory.DataFactory;

import org.springframework.beans.factory.annotation.Required;

import ca.odell.glazedlists.BasicEventList;
import ca.odell.glazedlists.CompositeList;
import ca.odell.glazedlists.EventList;

/**
 * Abstract field descriptor for fields that are based on a list of objects.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 * @param <E>
 *            Type of object for the list
 */
public abstract class AbstractListPropertyFieldDescriptor<V, E> extends
		AbstractPropertyFieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private DataFactory<EventList<E>> itemListFactory;
	private boolean itemListAddNull = false;
	private ListCellRenderer renderer;
	private LabelInfoProvider labelInfoProvider = new DefaultLabelInfoProvider();

	/**
	 * Gets the list model.
	 * 
	 * @param view
	 *            the view
	 * 
	 * @return the list model
	 */
	protected EventList<E> getListModel(GUIView<V> view) {
		EventList<E> sourceList = getInitialListValueModel(view);
		// Adding null ?
		if (this.itemListAddNull) {
			// Converts to event list
			CompositeList<E> compositeList = createListWithNull(sourceList);
			// OK
			return compositeList;
		}
		// No
		else {
			return sourceList;
		}
	}

	/**
	 * Gets the initial list value model.
	 * 
	 * @param view
	 *            the view
	 * 
	 * @return the initial list value model
	 */
	protected EventList<E> getInitialListValueModel(GUIView<V> view) {
		// Gets the view model
		V viewModel = view.getViewData();
		// Gets the list model
		EventList<E> sourceList = this.itemListFactory.create(viewModel);
		// OK
		return sourceList;
	}

	/**
	 * Creates the list with null.
	 * 
	 * @param sourceList
	 *            the source list
	 * 
	 * @return the composite list< object>
	 */
	protected CompositeList<E> createListWithNull(EventList<E> sourceList) {
		// Creates a composite
		CompositeList<E> compositeList = new CompositeList<E>(sourceList
				.getPublisher(), sourceList.getReadWriteLock());
		// Null list
		BasicEventList<E> nullList = new BasicEventList<E>(sourceList
				.getPublisher(), sourceList.getReadWriteLock());
		nullList.add(null);
		// Composition
		compositeList.addMemberList(nullList);
		compositeList.addMemberList(sourceList);
		return compositeList;
	}

	/**
	 * Checks if a <code>null</code> entry is added at the beginning of the
	 * list.
	 * 
	 * @return <code>true</code> if a <code>null</code> entry is added at the
	 *         beginning of the list
	 */
	public boolean isItemListAddNull() {
		return this.itemListAddNull;
	}

	/**
	 * Sets if a <code>null</code> entry is added at the beginning of the list
	 * 
	 * @param itemListAddNull
	 *            <code>true</code> if a <code>null</code> entry is added at the
	 *            beginning of the list
	 */
	public void setItemListAddNull(boolean itemListAddNull) {
		this.itemListAddNull = itemListAddNull;
	}

	/**
	 * Gets the item list factory.
	 * 
	 * @return the item list factory
	 */
	public DataFactory<EventList<E>> getItemListFactory() {
		return this.itemListFactory;
	}

	/**
	 * Sets the item list factory.
	 * 
	 * @param itemListFactory
	 *            the new item list factory
	 */
	@Required
	public void setItemListFactory(DataFactory<EventList<E>> itemListFactory) {
		this.itemListFactory = itemListFactory;
	}

	/**
	 * Gets the cell renderer for the list
	 * 
	 * @return Renderer
	 */
	public ListCellRenderer getRenderer() {
		return this.renderer;
	}

	/**
	 * Sets the cell renderer for the list
	 * 
	 * @param renderer
	 *            Renderer
	 */
	public void setRenderer(ListCellRenderer renderer) {
		this.renderer = renderer;
	}

	/**
	 * Gets the {@link LabelInfoProvider} used to render the cell. If the
	 * {@link #getRenderer() renderer} is defined, this property is ignored
	 * 
	 * @return Label info provider
	 */
	public LabelInfoProvider getLabelInfoProvider() {
		return this.labelInfoProvider;
	}

	/**
	 * Sets the {@link LabelInfoProvider} used to render the cell. It is set by
	 * default to {@link DefaultLabelInfoProvider}.
	 * 
	 * @param labelInfoProvider
	 *            Label info provider
	 * @see #getLabelInfoProvider()
	 */
	public void setLabelInfoProvider(LabelInfoProvider labelInfoProvider) {
		this.labelInfoProvider = labelInfoProvider;
	}

	/**
	 * Gets the actual cell renderer to use.
	 * <p>
	 * If the {@link #getRenderer() renderer} is set, it is used.
	 * <p>
	 * If the {@link #getLabelInfoProvider() labelInfoProvider} is set, a
	 * {@link LabelInfoListCellRenderer} is created and used.
	 * <p>
	 * Otherwise, a {@link DefaultListCellRenderer} is used.
	 * 
	 * @return A list cell renderer
	 */
	protected ListCellRenderer getActualCellRenderer() {
		if (this.renderer != null) {
			return this.renderer;
		} else if (this.labelInfoProvider != null) {
			return new LabelInfoListCellRenderer(this.labelInfoProvider);
		} else {
			return new DefaultListCellRenderer();
		}
	}
}
