package net.sf.doolin.gui.field.list;

import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.JComboBox;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import net.sf.doolin.gui.field.Field;
import net.sf.doolin.gui.field.support.SimpleField;
import net.sf.doolin.gui.swing.CompletionSupport;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.util.Utils;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.TextFilterator;
import ca.odell.glazedlists.matchers.TextMatcherEditor;
import ca.odell.glazedlists.swing.AutoCompleteSupport;
import ca.odell.glazedlists.swing.EventComboBoxModel;

/**
 * {@link JComboBox} field descriptor.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 * @param <E>
 *            Type of object in the item list
 */
public class FieldCombo<V, E> extends AbstractListPropertyFieldDescriptor<V, E> {

	private static final long serialVersionUID = 1L;

	private final static Logger log = LoggerFactory.getLogger(FieldCombo.class);

	private CompletionSupport<E> completionSupport;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Field<V> createField(final GUIView<V> view) {
		final JComboBox cbo;
		// Gets the source list
		final EventList<E> sourceList = getListModel(view);
		// Completion enabled
		if (this.completionSupport != null) {
			// Creates the combo-box
			cbo = new JComboBox();
			// Filterator
			TextFilterator<E> textFilterator = this.completionSupport
					.getOrCreateTextFilterator();
			// Auto-completion support
			AutoCompleteSupport<E> autoCompleteSupport = AutoCompleteSupport
					.install(cbo, sourceList, textFilterator,
							this.completionSupport.getFormat());
			// Strict mode
			autoCompleteSupport.setStrict(this.completionSupport.isStrict());
			// Filter mode
			String filterMode = this.completionSupport.getFilterMode();
			if (StringUtils.isNotBlank(filterMode)) {
				int mode = (Integer) Utils.getConstant(TextMatcherEditor.class,
						filterMode);
				autoCompleteSupport.setFilterMode(mode);
			}
		}
		// No completion
		else {
			// Creates the combo-box
			cbo = new JComboBox();
			// Model
			cbo.setModel(new EventComboBoxModel<E>(sourceList));
			// Renderer
			cbo.setRenderer(getActualCellRenderer());
		}
		// Selection flag
		final AtomicBoolean onSelection = new AtomicBoolean(false);
		// Initial value
		cbo.setSelectedItem(getProperty(view));
		// Selection -> model
		cbo.getModel().addListDataListener(new ListDataListener() {

			@Override
			public void contentsChanged(ListDataEvent e) {
				int min = e.getIndex0();
				int max = e.getIndex1();
				if (min == max && min == -1 && !onSelection.get()) {
					onSelection.set(true);
					try {
						Object item = cbo.getSelectedItem();
						log.debug(String.format("Selecting %s", item));
						setProperty(view, item);
					} finally {
						onSelection.set(false);
					}
				}
			}

			@Override
			public void intervalAdded(ListDataEvent e) {
			}

			@Override
			public void intervalRemoved(ListDataEvent e) {
			}

		});
		// Model -> Selection
		subscribe(view, new Runnable() {

			@Override
			public void run() {
				if (!onSelection.get()) {
					onSelection.set(true);
					try {
						Object item = getProperty(view);
						log.debug(String.format("Model selection %s", item));
						cbo.setSelectedItem(item);
					} finally {
						onSelection.set(false);
					}
				}
			}
		});
		// OK
		return new SimpleField<V>(view, this, cbo);
	}

	/**
	 * Gets the completion support for this combo box.
	 * 
	 * @return Completion support or <code>null</code> is no completion must be
	 *         done on the combo box.
	 */
	public CompletionSupport<E> getCompletionSupport() {
		return this.completionSupport;
	}

	/**
	 * Sets the completion support for this combo box.
	 * 
	 * @param completionSupport
	 *            Completion support or <code>null</code> is no completion must
	 *            be done on the combo box.
	 */
	public void setCompletionSupport(CompletionSupport<E> completionSupport) {
		this.completionSupport = completionSupport;
	}
}
