package net.sf.doolin.gui.field.table;

import java.util.List;

import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.action.path.list.ActionPathList;
import net.sf.doolin.gui.action.path.toolbar.ToolBar;
import net.sf.doolin.gui.field.AbstractFieldDescriptor;
import net.sf.doolin.gui.field.Field;
import net.sf.doolin.gui.field.table.header.CustomHeaderFactory;
import net.sf.doolin.gui.field.table.header.GUITableHeaderFactory;
import net.sf.doolin.gui.field.table.header.support.DefaultGUITableHeaderFactory;
import net.sf.doolin.gui.field.table.style.TableStyle;
import net.sf.doolin.gui.field.table.support.GUITableField;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.util.factory.DataFactory;

import org.springframework.beans.factory.annotation.Required;

import ca.odell.glazedlists.matchers.MatcherEditor;

/**
 * Table field descriptor.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 * @param <E>
 *            Type of item in the table
 */
public class FieldTable<V, E> extends AbstractFieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private ITableEventListFactory<E> itemListFactory;
	private ColumnGenerator<V, E> columnGenerator;
	private TableStyle tableStyle;
	private boolean itemChanges;
	private ActionPathList tableMenu;
	private ActionPathList tableToolBar;
	private ActionPathList viewToolBar;
	private ToolBar windowToolBar;
	private GUIAction doubleClickAction;
	private List<GUIAction> tableActions;
	private DataFactory<MatcherEditor<E>> matcherEditorFactory;
	private String selectionPropertyPath;
	private TableSelectionMode selectionMode = TableSelectionMode.SINGLE;
	private boolean showColumns = true;
	private CustomHeaderFactory<V, E> customHeaderFactory = null;
	private GUITableHeaderFactory<V, E> headerFactory = new DefaultGUITableHeaderFactory<V, E>();

	/**
	 * {@inheritDoc}.
	 * 
	 * @param view
	 *            the view
	 * 
	 * @return the field
	 */
	@Override
	public Field<V> createField(GUIView<V> view) {
		return new GUITableField<V, E>(view, this);
	}

	/**
	 * Gets the column generator.
	 * 
	 * @return the column generator
	 */
	public ColumnGenerator<V, E> getColumnGenerator() {
		return this.columnGenerator;
	}

	/**
	 * @return the customHeaderFactory
	 */
	public CustomHeaderFactory<V, E> getCustomHeaderFactory() {
		return this.customHeaderFactory;
	}

	/**
	 * Gets the action to execute when a row is double-clicked.
	 * 
	 * @return Action or <code>null</code>
	 */
	public GUIAction getDoubleClickAction() {
		return this.doubleClickAction;
	}

	/**
	 * @return the headerFactory
	 */
	public GUITableHeaderFactory<V, E> getHeaderFactory() {
		return this.headerFactory;
	}

	public ITableEventListFactory<E> getItemListFactory() {
		return this.itemListFactory;
	}

	/**
	 * Returns the factory for the matcher editor used to filter the items in
	 * the table.
	 * 
	 * @return Factory for the matcher editor
	 */
	public DataFactory<MatcherEditor<E>> getMatcherEditorFactory() {
		return this.matcherEditorFactory;
	}

	/**
	 * Gets the selection mode for the table rows
	 * 
	 * @return Selection mode
	 */
	public TableSelectionMode getSelectionMode() {
		return this.selectionMode;
	}

	/**
	 * Gets the path to the property that contains the selection of the table.
	 * If multiple selections are allowed, this property must hold a
	 * {@link List}, else if must hold a &lt;E&gt; type.
	 * 
	 * @return Path to the property
	 */
	public String getSelectionPropertyPath() {
		return this.selectionPropertyPath;
	}

	/**
	 * Gets the list of actions to register together with the table for the
	 * availability of their shortcuts.
	 * 
	 * @return List of actions
	 */
	public List<GUIAction> getTableActions() {
		return this.tableActions;
	}

	/**
	 * Gets the definition for the contextual menu of the table
	 * 
	 * @return Menu definition
	 */
	public ActionPathList getTableMenu() {
		return this.tableMenu;
	}

	/**
	 * Gets the style.
	 * 
	 * @return the style
	 */
	public TableStyle getTableStyle() {
		return this.tableStyle;
	}

	/**
	 * Gets the tool bar definition for the table-level tool bar.
	 * 
	 * @return Tool bar definition
	 */
	public ActionPathList getTableToolBar() {
		return this.tableToolBar;
	}

	/**
	 * Gets the tool bar definition for the view-level tool bar.
	 * 
	 * @return Tool bar definition
	 */
	public ActionPathList getViewToolBar() {
		return this.viewToolBar;
	}

	/**
	 * Gets the tool bar definition for the window-level tool bar.
	 * 
	 * @return Tool bar definition
	 */
	public ToolBar getWindowToolBar() {
		return this.windowToolBar;
	}

	/**
	 * Checks if the table must listen to changes in the row items.
	 * 
	 * @return <code>true</code> if the table must listen to the changes
	 * @see #setItemChanges(boolean)
	 */
	public boolean isItemChanges() {
		return this.itemChanges;
	}

	/**
	 * Checks if the column headers are displayed.
	 * 
	 * @return <code>true</code> if the column headers are displayed.
	 */
	public boolean isShowColumns() {
		return this.showColumns;
	}

	/**
	 * Sets the generator for the columns of the tabler.
	 * 
	 * @param columnGenerator
	 *            the new column generator
	 */
	@Required
	public void setColumnGenerator(ColumnGenerator<V, E> columnGenerator) {
		this.columnGenerator = columnGenerator;
	}

	/**
	 * @param customHeaderFactory
	 *            the customHeaderFactory to set
	 */
	public void setCustomHeaderFactory(
			CustomHeaderFactory<V, E> customHeaderFactory) {
		this.customHeaderFactory = customHeaderFactory;
	}

	/**
	 * Sets the action to execute when a row is double-clicked.
	 * 
	 * @param doubleClickAction
	 *            Action or <code>null</code>
	 */
	public void setDoubleClickAction(GUIAction doubleClickAction) {
		this.doubleClickAction = doubleClickAction;
	}

	/**
	 * @param headerFactory
	 *            the headerFactory to set
	 */
	public void setHeaderFactory(GUITableHeaderFactory<V, E> headerFactory) {
		this.headerFactory = headerFactory;
	}

	/**
	 * Sets if the table must listen to changes in the row items. If set to
	 * <code>true</code> (it is set to <code>false</code> by default), each time
	 * a bean is updated that belongs to the table rows, the corresponding row
	 * will be updated.
	 * 
	 * @param itemChanges
	 *            <code>true</code> if the table must listen to the changes
	 */
	public void setItemChanges(boolean itemChanges) {
		this.itemChanges = itemChanges;
	}

	public void setItemListFactory(ITableEventListFactory<E> itemListFactory) {
		this.itemListFactory = itemListFactory;
	}

	/**
	 * Sets the factory for the matcher editor used to filter the items in the
	 * table.
	 * 
	 * @param matcherEditorFactory
	 *            Factory for the matcher editor
	 */
	public void setMatcherEditorFactory(
			DataFactory<MatcherEditor<E>> matcherEditorFactory) {
		this.matcherEditorFactory = matcherEditorFactory;
	}

	/**
	 * Sets the selection mode for the table rows
	 * 
	 * @param selectionMode
	 *            Selection mode
	 */
	public void setSelectionMode(TableSelectionMode selectionMode) {
		this.selectionMode = selectionMode;
	}

	/**
	 * Sets the path to the property that contains the selection of the table.
	 * If multiple selections are allowed, this property must hold a
	 * {@link List}, else if must hold a &lt;E&gt; type.
	 * 
	 * @param selectionPropertyPath
	 *            Path to the property or <code>null</code> if no selection is
	 *            to be saved
	 */
	public void setSelectionPropertyPath(String selectionPropertyPath) {
		this.selectionPropertyPath = selectionPropertyPath;
	}

	/**
	 * Sets if the column headers are displayed.
	 * 
	 * @param showColumns
	 *            <code>true</code> if the column headers are displayed.
	 */
	public void setShowColumns(boolean showColumns) {
		this.showColumns = showColumns;
	}

	/**
	 * Sets the list of actions to register together with the table for the
	 * availability of their shortcuts.
	 * 
	 * @param tableActions
	 *            List of actions
	 */
	public void setTableActions(List<GUIAction> tableActions) {
		this.tableActions = tableActions;
	}

	/**
	 * Sets the definition for the contextual menu of the table
	 * 
	 * @param tableMenu
	 *            Menu definition
	 */
	public void setTableMenu(ActionPathList tableMenu) {
		this.tableMenu = tableMenu;
	}

	/**
	 * Sets the default style to apply on the cells.
	 * 
	 * @param style
	 *            the new style
	 */
	public void setTableStyle(TableStyle style) {
		this.tableStyle = style;
	}

	/**
	 * Sets the tool bar definition for the table-level tool bar.
	 * 
	 * @param tableToolBar
	 *            Tool bar definition
	 */
	public void setTableToolBar(ActionPathList tableToolBar) {
		this.tableToolBar = tableToolBar;
	}

	/**
	 * Sets the tool bar definition for the view-level tool bar.
	 * 
	 * @param viewToolBar
	 *            Tool bar definition
	 */
	public void setViewToolBar(ActionPathList viewToolBar) {
		this.viewToolBar = viewToolBar;
	}

	/**
	 * Sets the tool bar definition for the window-level tool bar.
	 * 
	 * @param windowToolBar
	 *            Tool bar definition
	 */
	public void setWindowToolBar(ToolBar windowToolBar) {
		this.windowToolBar = windowToolBar;
	}

}
