package net.sf.doolin.gui.field.tree.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.bus.support.SubscriberValidator;
import net.sf.doolin.gui.field.tree.Children;
import net.sf.doolin.gui.field.tree.Node;
import net.sf.doolin.gui.swing.LabelInfo;
import net.sf.doolin.gui.swing.LabelInfoProvider;
import net.sf.doolin.util.Pair;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.event.ListEvent;
import ca.odell.glazedlists.event.ListEventListener;

import com.google.common.base.Predicate;

public class GUITreeNode extends DefaultMutableTreeNode {

	private static final Logger logger = LoggerFactory
			.getLogger(GUITreeNode.class);

	private final Node def;
	private final List<Pair<EventList<Object>, ListEventListener<Object>>> eventListListeners = new ArrayList<Pair<EventList<Object>, ListEventListener<Object>>>();

	/**
	 * Constructor for a node.
	 * 
	 * @param validator
	 *            Validator to be used for the bus subscriptions
	 * @param model
	 *            Owner
	 * @param def
	 *            Node definition
	 * @param bean
	 *            Associated user object
	 */
	public GUITreeNode(SubscriberValidator validator, final GUITreeModel model,
			final Node def, final Object bean) {
		super(bean);
		this.def = def;
		// Listeners for the bean properties
		if (def.isChangeSupport()) {
			PropertyChangeSupport.subscribe(validator, bean, null,
					new Runnable() {

						@Override
						public void run() {
							model.nodeChanged(GUITreeNode.this);
						}

						@Override
						public String toString() {
							return String.format(
									"Updates the node %s for bean %s", def
											.getId(), bean);
						}

					});
		}
		// Children
		fill(validator, model, bean);
	}

	/**
	 * Creates a node for a child.
	 * 
	 * @param validator
	 *            Validator to be used for the bus subscriptions
	 * @param model
	 *            Owner
	 * @param childNode
	 *            Definition for the child node
	 * @param childBean
	 *            Bean associated to the child node
	 * @return Node for the child
	 */
	protected GUITreeNode createChildTreeNode(SubscriberValidator validator,
			GUITreeModel model, Node childNode, Object childBean) {
		return new GUITreeNode(validator, model, childNode, childBean);
	}

	/**
	 * Initializes a node
	 * 
	 * @param validator
	 *            Validator to be used for the bus subscriptions
	 * @param model
	 *            Owner
	 * @param bean
	 *            Associated bean
	 */
	protected void fill(final SubscriberValidator validator,
			final GUITreeModel model, final Object bean) {
		// Clears of listeners
		for (Pair<EventList<Object>, ListEventListener<Object>> pair : this.eventListListeners) {
			EventList<Object> list = pair.getFirstValue();
			ListEventListener<Object> listener = pair.getSecondValue();
			try {
				logger.debug("Removing listener {} from list {}", listener,
						list);
				list.removeListEventListener(listener);
			} catch (IllegalArgumentException th) {
				// Ignoring the fact that the listener may be invalid
				logger.warn("Could not remove listener {} from list {}",
						listener, list);
			}
		}
		this.eventListListeners.clear();
		// Clears of children
		removeAllChildren();
		// For all children definitions
		List<Children> childrenList = this.def.getChildren();
		if (childrenList == null || childrenList.isEmpty()) {
			setAllowsChildren(false);
		} else {
			setAllowsChildren(true);
			for (Children children : childrenList) {
				// Child definition
				Node childNode = model.getNode(children.getId());
				// Get the list of children
				Collection<?> childBeanList = getChildBeanList(bean, children);
				if (childBeanList != null) {
					fillNodeFromList(validator, model, childNode, childBeanList);
					// If the list of children can be listened to...
					if (childBeanList instanceof EventList<?>) {
						@SuppressWarnings("unchecked")
						EventList<Object> childBeanEventList = (EventList<Object>) childBeanList;
						ListEventListener<Object> eventListener = new ListEventListener<Object>() {

							@Override
							public void listChanged(ListEvent<Object> e) {
								// Deletes previous list of nodes
								removeAllChildren();
								// Fills the list again
								fill(validator, model, bean);
								// Updates the node
								model.nodeStructureChanged(GUITreeNode.this);
							}
						};
						logger.debug("Adding listener {} to list {}",
								eventListener, childBeanEventList);
						childBeanEventList.addListEventListener(eventListener);
						// Registers the listener
						this.eventListListeners
								.add(new Pair<EventList<Object>, ListEventListener<Object>>(
										childBeanEventList, eventListener));
					}
				}
			}
		}
	}

	protected void fillNodeFromList(SubscriberValidator validator,
			GUITreeModel model, Node childNode, Collection<?> childBeanList) {
		// Fills children
		for (Object childBean : childBeanList) {
			GUITreeNode childTreeNode = createChildTreeNode(validator, model,
					childNode, childBean);
			add(childTreeNode);
		}
	}

	/**
	 * Gets the list of beans which are associated to a children definition
	 * 
	 * @param bean
	 *            Object associated to the parent node
	 * @param children
	 *            Children definition
	 * @return List of objects that define the children
	 */
	protected Collection<?> getChildBeanList(Object bean, Children children) {
		return children.getChildBeans(bean);
	}

	public Node getDefinition() {
		return this.def;
	}

	/**
	 * Get the label information for this node.
	 * 
	 * @return Label information for this node.
	 * @see Node#getLabelProvider()
	 * @see LabelInfoProvider#getLabelIcon(Object)
	 */
	public LabelInfo getLabelInfo() {
		return this.def.getLabelProvider().getLabelIcon(getUserObject());
	}

	public TreePath getTreePath() {
		List<TreeNode> list = new ArrayList<TreeNode>();

		// Add all nodes to list
		TreeNode node = this;
		while (node != null) {
			list.add(node);
			node = node.getParent();
		}
		Collections.reverse(list);

		// Convert array of nodes to TreePath
		return new TreePath(list.toArray());
	}

	/**
	 * Looks for a child node that complies with the given
	 * <code>predicate</code>.
	 * 
	 * @param onlyImmediates
	 *            <code>true</code> if only immediate children must be looked
	 *            for. It <code>false</code>, the whole tree of descendants is
	 *            looked for.
	 * @param predicate
	 *            The predicate used to identify the node
	 * @return Node that complies with the predicate or <code>null</code> if
	 *         none has been found.
	 */
	public GUITreeNode lookForNode(boolean onlyImmediates,
			Predicate<GUITreeNode> predicate) {
		// Gets all children (if any)
		if (getChildCount() > 0) {
			@SuppressWarnings("unchecked")
			Enumeration<GUITreeNode> e = children();
			while (e.hasMoreElements()) {
				GUITreeNode node = e.nextElement();
				if (predicate.apply(node)) {
					// Found it
					return node;
				} else if (!onlyImmediates) {
					GUITreeNode result = node.lookForNode(false, predicate);
					// Found something under this child
					if (result != null) {
						return result;
					}
				}
			}
			// Nothing has been found
			return null;
		} else {
			return null;
		}
	}

}
