package net.sf.doolin.gui.layout;

import java.util.Collections;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.swing.JComponent;
import javax.swing.JPanel;

import org.apache.commons.lang.StringUtils;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

/**
 * Layout based on {@link com.jgoodies.forms.layout.FormLayout}.
 * 
 * @author Damien Coraboeuf
 */
public class FormLayoutService implements LayoutService {

	private boolean defaultBorder = true;

	private String columnSpecs = "default:grow";

	private String columnGroups;

	private String rowSpecs;

	private String rowSpec = "default";

	private String rowGap = "3dlu";

	private int rowCount = 1;

	private String rowGroups;

	/**
	 * Constructor with default values
	 */
	public FormLayoutService() {
	}

	/**
	 * Constructor
	 * 
	 * @param columnSpecs
	 *            Column specifications
	 * @param rowSpecs
	 *            Row specifications
	 * @see #setColumnSpecs(String)
	 * @see #setRowSpecs(String)
	 */
	public FormLayoutService(String columnSpecs, String rowSpecs) {
		this.columnSpecs = columnSpecs;
		this.rowSpecs = rowSpecs;
	}

	/**
	 * Initialisation of properties.
	 * <p>
	 * If <code>rowSpecs</code> property has not been set, it is computed using
	 * <code>rowSpec</code>, <code>rowGap</code> and <code>rowCount</code>
	 * using:
	 * 
	 * <pre>
	 * rowSpecs := rowSpec &quot;,&quot; rowGap &quot;,&quot; rowSpec &quot;,&quot; rowGap ... (rowCount times)
	 * </pre>
	 */
	@PostConstruct
	public void postConstruct() {
		if (StringUtils.isBlank(this.rowSpecs)) {
			if (this.rowCount == 0) {
				throw new IllegalStateException(
						"If rowSpecs is blank, rowCount must be > 0");
			} else {
				List<String> specs = Collections.nCopies(this.rowCount,
						this.rowSpec);
				this.rowSpecs = StringUtils
						.join(specs, "," + this.rowGap + ",");
			}
		}
	}

	@Override
	public void add(JPanel panel, JComponent component, Object constraint) {
		panel.add(component, constraint);
	}

	@Override
	public void init(JPanel panel) {
		FormLayout layout = new FormLayout(this.columnSpecs, this.rowSpecs);
		DefaultFormBuilder builder = new DefaultFormBuilder(layout, panel);
		if (this.defaultBorder) {
			builder.setDefaultDialogBorder();
		}
		// Groups
		if (StringUtils.isNotBlank(this.columnGroups)) {
			layout.setColumnGroups(parseGroups(this.columnGroups));
		}
		if (StringUtils.isNotBlank(this.rowGroups)) {
			layout.setRowGroups(parseGroups(this.rowGroups));
		}
	}

	/**
	 * Parses a row or column group definition.
	 * <p>
	 * Groups are comma-separated lists of integers. And the group definition is
	 * a "|"-separated list of groups.
	 * <p>
	 * For example,<br>
	 * <code>3,4|5,7</code><br>
	 * stands for<br>
	 * <code>{{3,4},{5,7}}</code>
	 * 
	 * @param group
	 *            Group definition
	 * 
	 * @return Group
	 */
	public int[][] parseGroups(String group) {
		if (StringUtils.isBlank(group)) {
			return new int[0][];
		} else {
			String[] groups = StringUtils.split(group, "|");
			int[][] result = new int[groups.length][];
			for (int i = 0; i < groups.length; i++) {
				String groupTokens = groups[i];
				String[] groupTokenArray = StringUtils.split(groupTokens, ",");
				int[] groupArray = new int[groupTokenArray.length];
				for (int j = 0; j < groupTokenArray.length; j++) {
					String groupToken = groupTokenArray[j];
					int groupIndex = Integer.parseInt(groupToken);
					groupArray[j] = groupIndex;
				}
				result[i] = groupArray;
			}
			return result;
		}
	}

	/**
	 * Gets the column specifications.
	 * 
	 * @return the column specifications
	 * @see FormLayout
	 */
	public String getColumnSpecs() {
		return this.columnSpecs;
	}

	/**
	 * Sets the column specifications.
	 * 
	 * @param columnSpecs
	 *            the new column specifications
	 * @see FormLayout
	 */
	public void setColumnSpecs(String columnSpecs) {
		this.columnSpecs = columnSpecs;
	}

	/**
	 * Gets the column groups.
	 * 
	 * @return the column groups
	 * @see #parseGroups(String)
	 */
	public String getColumnGroups() {
		return this.columnGroups;
	}

	/**
	 * Sets the column groups.
	 * 
	 * @param columnGroups
	 *            the new column groups
	 * @see #parseGroups(String)
	 */
	public void setColumnGroups(String columnGroups) {
		this.columnGroups = columnGroups;
	}

	/**
	 * Gets the row specifications.
	 * 
	 * @return the row specifications
	 * @see FormLayout
	 */
	public String getRowSpecs() {
		return this.rowSpecs;
	}

	/**
	 * Sets the row specifications.
	 * 
	 * @param rowSpecs
	 *            the new row specifications
	 * @see FormLayout
	 */
	public void setRowSpecs(String rowSpecs) {
		this.rowSpecs = rowSpecs;
	}

	/**
	 * Gets the row groups.
	 * 
	 * @return the row groups
	 * @see #parseGroups(String)
	 */
	public String getRowGroups() {
		return this.rowGroups;
	}

	/**
	 * Sets the row groups.
	 * 
	 * @param rowGroups
	 *            the new row groups
	 * @see #parseGroups(String)
	 */
	public void setRowGroups(String rowGroups) {
		this.rowGroups = rowGroups;
	}

	/**
	 * Gets the specification for one row.
	 * 
	 * @return Specification
	 * @see #postConstruct()
	 */
	public String getRowSpec() {
		return this.rowSpec;
	}

	/**
	 * Sets the specification for one row.
	 * 
	 * @param rowSpec
	 *            Specification
	 * @see #postConstruct()
	 */
	public void setRowSpec(String rowSpec) {
		this.rowSpec = rowSpec;
	}

	/**
	 * Gets the row gap.
	 * 
	 * @return the row gap
	 * @see #postConstruct()
	 */
	public String getRowGap() {
		return this.rowGap;
	}

	/**
	 * Sets the row gap.
	 * 
	 * @param rowGap
	 *            the new row gap
	 * @see #postConstruct()
	 */
	public void setRowGap(String rowGap) {
		this.rowGap = rowGap;
	}

	/**
	 * Gets the row count.
	 * 
	 * @return the row count
	 * @see #postConstruct()
	 */
	public int getRowCount() {
		return this.rowCount;
	}

	/**
	 * Sets the row count.
	 * 
	 * @param rowCount
	 *            the new row count
	 * @see #postConstruct()
	 */
	public void setRowCount(int rowCount) {
		this.rowCount = rowCount;
	}

	/**
	 * Checks if is default border.
	 * 
	 * @return true, if is default border
	 */
	public boolean isDefaultBorder() {
		return this.defaultBorder;
	}

	/**
	 * Sets the default border.
	 * 
	 * @param defaultBorder
	 *            the new default border
	 */
	public void setDefaultBorder(boolean defaultBorder) {
		this.defaultBorder = defaultBorder;
	}

}
