package net.sf.doolin.gui.service.info;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Locale;

/**
 * Default implementation for an {@link ErrorMessageService}.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class DefaultErrorMessageService implements ErrorMessageService {

	private ExceptionTranslator exceptionTranslator = new DefaultExceptionTranslator();

	protected boolean excludes(Throwable th) {
		return (th.getClass().getName()
				.equals(RuntimeException.class.getName()) && th.getCause() != null);
	}

	/**
	 * Computes the error message.
	 * <p>
	 * If <code>e</code> is {@link net.sf.jstring.Localizable}, its
	 * {@linkplain net.sf.jstring.Localizable#getLocalizedMessage(Locale) localized message} is
	 * called using the {@linkplain Locale#getDefault() default locale}.
	 * <p>
	 * In any other case, we take the list of recursive non-empty messages along
	 * the exception chain and concatenate them using the platform separator.
	 */
	@Override
	public String getErrorMessage(Throwable e) {
		// Gets the exception chain
		Throwable[] chain = ExceptionUtils.getThrowables(e);
		// For each exception
		for (Throwable th : chain) {
			// Filters on exception
			if (excludes(th)) {
				continue;
			}
			// Gets its message
			String message = getIndividualMessage(th);
			// If not empty, OK
			if (StringUtils.isNotBlank(message)) {
				return message;
			}
		}
		// No valid message is left, just returns the root
		return e.getClass().getSimpleName();
	}

	public ExceptionTranslator getExceptionTranslator() {
		return this.exceptionTranslator;
	}

	protected String getIndividualMessage(Throwable th) {
		return this.exceptionTranslator.getMessage(th);
	}

	@Autowired(required = false)
	public void setExceptionTranslator(ExceptionTranslator exceptionTranslator) {
		this.exceptionTranslator = exceptionTranslator;
	}

}
