/*
 * Created on Aug 6, 2007
 */
package net.sf.doolin.gui.swing;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JComponent;

import net.sf.doolin.gui.service.GUIPreferences;
import net.sf.doolin.util.PatternFileFilter;

import org.apache.commons.lang.StringUtils;

/**
 * File selection component.
 * 
 * @author Damien Coraboeuf
 */
public class JFile extends JComponent {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * <code>file</code> property that contains the selected {@link File}.
	 */
	public static final String PROPERTY_FILE = "file";

	private boolean directory = false;

	private FileBrowsingMode mode;

	private String filter;

	private String filterDescription;

	private JButton browseButton;

	private final IFilePath filePath;
	private final JComponent pathComponent;

	private File file;

	private Color pathTextDefaultColor;

	private GUIPreferences preferences;

	/**
	 * Constructor with a {@link TextFilePath} implementation for the file path.
	 */
	public JFile() {
		this(new TextFilePath());
	}

	/**
	 * Constructor for a {@link JFile} that interacts with the preferences
	 * 
	 * @param preferences
	 *            Preferences
	 */
	public JFile(GUIPreferences preferences) {
		this();
		this.preferences = preferences;
	}

	/**
	 * Constructor for a {@link JFile} that interacts with the preferences and
	 * which is based on a given file path implementation
	 * 
	 * @param preferences
	 *            Preferences
	 * @param filePath
	 *            File path implementation
	 */
	public JFile(GUIPreferences preferences, IFilePath filePath) {
		this(filePath);
		this.preferences = preferences;
	}

	/**
	 * Constructor based on a given file path implementation
	 * 
	 * @param filePath
	 *            File path implementation
	 */
	public JFile(IFilePath filePath) {
		setLayout(new BorderLayout());
		// Text
		this.filePath = filePath;
		this.pathComponent = filePath.getComponent();
		this.pathTextDefaultColor = this.pathComponent.getForeground();
		this.filePath.addFilePathListener(new FilePathListener() {

			@Override
			public void filePathChanged(String newPath) {
				onPathEdited();
			}

		});
		add(this.pathComponent, BorderLayout.CENTER);
		// Browse button
		this.browseButton = new JButton("...");
		this.browseButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				onBrowse();
			}

		});
		add(this.browseButton, BorderLayout.LINE_END);
	}

	/**
	 * Gets the file.
	 * 
	 * @return the file
	 */
	public File getFile() {
		return this.file;
	}

	/**
	 * Gets the filter.
	 * 
	 * @return the filter
	 */
	public String getFilter() {
		return this.filter;
	}

	/**
	 * Gets the filter description.
	 * 
	 * @return the filter description
	 */
	public String getFilterDescription() {
		return this.filterDescription;
	}

	/**
	 * Gets the mode.
	 * 
	 * @return the mode
	 */
	public FileBrowsingMode getMode() {
		return this.mode;
	}

	/**
	 * Checks if is directory.
	 * 
	 * @return true, if is directory
	 */
	public boolean isDirectory() {
		return this.directory;
	}

	/**
	 * This method is called when the user has clicked on the browse button.
	 */
	protected void onBrowse() {
		FileBrowser browser = new FileBrowser(this.preferences);
		browser.setModeDirectory(this.directory);
		browser.setModeSave(this.mode == FileBrowsingMode.SAVE);
		browser.setFileFilter(this.filter);
		browser.setFileDescription(this.filterDescription);
		browser.setFile(this.file);
		File f = browser.browse(this);
		if (f != null) {
			updateFile(f);
			this.filePath.setFile(this.file);
			this.pathComponent.setForeground(this.pathTextDefaultColor);
		}
	}

	/**
	 * This method is called when the path editor has lost its focus.
	 */
	protected void onPathEdited() {
		String path = this.filePath.getFilePath();
		if (StringUtils.isBlank(path)) {
			updateFile(null);
		} else {
			File f = new File(path);
			// Checks the conformity of the file to the filter
			boolean isSave = this.mode == FileBrowsingMode.SAVE;
			boolean ok;
			if (this.directory) {
				if (isSave) {
					ok = true;
				} else {
					ok = f.exists() && f.isDirectory();
				}
			} else {
				if (isSave) {
					PatternFileFilter patternFileFilter = new PatternFileFilter(
							this.filter);
					ok = patternFileFilter.accept(f);
				} else {
					PatternFileFilter patternFileFilter = new PatternFileFilter(
							this.filter);
					ok = patternFileFilter.accept(f) && f.exists()
							&& f.isFile() && f.canRead();
				}
			}
			if (ok) {
				updateFile(f);
				this.pathComponent.setForeground(this.pathTextDefaultColor);
			} else {
				// updateFile(null);
				this.pathComponent.setForeground(Color.red);
			}
		}
	}

	/**
	 * Sets the directory.
	 * 
	 * @param directory
	 *            the new directory
	 */
	public void setDirectory(boolean directory) {
		this.directory = directory;
	}

	/**
	 * Sets the file.
	 * 
	 * @param f
	 *            the new file
	 */
	public void setFile(File f) {
		this.file = f;
		// Text
		if (this.file != null) {
			this.filePath.setFile(this.file);
		} else {
			this.filePath.setFile(null);
		}
	}

	/**
	 * Sets if the component is enabled or not.
	 * 
	 * @param enabled
	 *            Enabled flag
	 */
	@Override
	public void setEnabled(boolean enabled) {
		// Default
		super.setEnabled(enabled);
		// Button
		this.browseButton.setVisible(enabled);
		// Path
		this.filePath.setEnabled(enabled);
	}

	/**
	 * Sets the filter.
	 * 
	 * @param filter
	 *            the new filter
	 */
	public void setFilter(String filter) {
		this.filter = filter;
	}

	/**
	 * Sets the filter description.
	 * 
	 * @param filterDescription
	 *            the new filter description
	 */
	public void setFilterDescription(String filterDescription) {
		this.filterDescription = filterDescription;
	}

	/**
	 * Sets the mode.
	 * 
	 * @param mode
	 *            the new mode
	 */
	public void setMode(FileBrowsingMode mode) {
		this.mode = mode;
	}

	/**
	 * Update file.
	 * 
	 * @param f
	 *            the f
	 */
	protected void updateFile(File f) {
		File oldFile = this.file;
		this.file = f;
		firePropertyChange(PROPERTY_FILE, oldFile, this.file);
	}

}
