/*
 * Created on Jul 17, 2007
 */
package net.sf.doolin.gui.swing;

import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.JLabel;

import org.apache.commons.lang.StringUtils;

/**
 * Computing of mnemonic information (character and index) from an encoded text.
 * 
 * The mnemonic is indicated by a &amp; character in the text. For example,
 * <code>E&amp;xit</code> defines <code>x</code> as a mnemonic for the
 * <code>Exit</code> text.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class MnemonicInfo {

	private char mnemonic = '\0';

	private int mnemonicIndex = -1;

	private String text;

	/**
	 * Constructor
	 * 
	 * @param name
	 *            Text
	 */
	public MnemonicInfo(String name) {
		if (StringUtils.isBlank(name)) {
			this.text = "";
		} else {
			int pos = getMnemonicIndex(name);
			if (pos >= 0) {
				this.mnemonic = name.charAt(pos + 1);
				this.text = name.substring(0, pos) + name.substring(pos + 1);
				this.mnemonicIndex = pos;
			} else {
				this.text = name;
			}
		}
	}

	/**
	 * Configures an action with this mnemonic information
	 * 
	 * @param action
	 *            Action to configure
	 * @see Action#putValue(String, Object)
	 * @see Action#NAME
	 * @see Action#MNEMONIC_KEY
	 * @see Action#DISPLAYED_MNEMONIC_INDEX_KEY
	 */
	public void configureAction(Action action) {
		action.putValue(Action.NAME, this.text);
		if (this.mnemonic != 0) {
			action.putValue(Action.MNEMONIC_KEY, Integer.valueOf(this.mnemonic));
			if (this.mnemonicIndex >= 0) {
				action.putValue(Action.DISPLAYED_MNEMONIC_INDEX_KEY,
						this.mnemonicIndex);
			}
		}
	}

	/**
	 * Configures a button with the mnemonic
	 * 
	 * @param button
	 *            Button to configure
	 * @see AbstractButton#setText(String)
	 * @see AbstractButton#setMnemonic(char)
	 * @see AbstractButton#setDisplayedMnemonicIndex(int)
	 */
	public void configureButton(AbstractButton button) {
		button.setText(this.text);
		if (this.mnemonic != 0) {
			button.setMnemonic(this.mnemonic);
			if (this.mnemonicIndex >= 0) {
				button.setDisplayedMnemonicIndex(this.mnemonicIndex);
			}
		}
	}

	/**
	 * Configures a label with this mnemonic information
	 * 
	 * @param label
	 *            Label to configure
	 * @see JLabel#setText(String)
	 * @see JLabel#setDisplayedMnemonicIndex(int)
	 */
	public void configureLabel(JLabel label) {
		label.setText(this.text);
		if (this.mnemonic != 0 && this.mnemonicIndex >= 0) {
			label.setDisplayedMnemonicIndex(this.mnemonicIndex);
		}
	}

	/**
	 * Returns the mnemonic character
	 * 
	 * @return mnemonic character or 0 is not defined
	 */
	public char getMnemonic() {
		return this.mnemonic;
	}

	/**
	 * Returns the mnemonic index
	 * 
	 * @return mnemonic index or -1 is not defined
	 */
	public int getMnemonicIndex() {
		return this.mnemonicIndex;
	}

	/**
	 * Text stripped from its mnemonic information
	 * 
	 * @return Text
	 */
	public String getText() {
		return this.text;
	}

	/**
	 * Returns the position of a valid &amp; character.
	 * 
	 * @param name
	 *            Text
	 * @return Position of the mnemonic
	 */
	protected int getMnemonicIndex(String name) {
		int pos = name.indexOf('&');
		if (pos >= 0 && pos < name.length() - 1) {
			char m = name.charAt(pos + 1);
			if (Character.isWhitespace(m)) {
				throw new IllegalArgumentException(
						"Mnemonic cannot be a space: " + name);
			} else {
				return pos;
			}
		}
		// Not found
		return -1;
	}

}
