/*
 * Copyright (c) 2003-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of JGoodies Karsten Lentzsch nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.sf.doolin.gui.validation;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.LayoutManager;
import java.awt.Point;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JViewport;
import javax.swing.text.JTextComponent;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.gui.swing.JFile;

/**
 * Can display validation feedback icons "over" a content panel. It observes a
 * ValidationResultModel and creates icon labels in a feedback layer of a
 * {@link JLayeredPane} on top of the content layer. To position the feedback
 * labels, the content pane is traversed and searched for text components that
 * match a validation message key in this panel's observed
 * ValidationResultModel.
 * <p>
 * 
 * <strong>Note:</strong> This panel doesn't reserve space for the portion used
 * to display the overlaid feedback components. It has been designed to not
 * change the layout of the wrapped content. Therefore you must reserve this
 * space, or in other words, you must ensure that the wrapped content provides
 * enough space to display the overlaid components. Since the current
 * implementation positions the overlay components in the lower left, just make
 * sure that there are about 6 pixel to the left and bottom of the input
 * components that can be marked.
 * <p>
 * 
 * This panel handles two event types:
 * <ol>
 * <li>the ValidationResultModel changes; in this case the set of visible
 * feedback components shall mark the input components that match the new
 * validation result. This is done by this class' internal
 * <code>ValidationResultChangeHandler</code> which in turn invokes
 * <code>#updateFeedbackComponents</code>.
 * <li>the content layout changes; the feedback components must then be
 * repositioned to reflect the position of the overlaid input components. This
 * is done by overriding <code>#validateTree</code> and invoking
 * <code>#repositionFeedBackComponents</code> after the child tree has been laid
 * out. The current simple but expensive implementation updates all components.
 * </ol>
 * <p>
 * 
 * TO DO: Check how the wrapping mechanism shall work with JSplitPanes,
 * JTabbedPanes and CardPanels. At least provide guidelines, how to wrap these
 * panel types, or how to handle these cases.
 * <p>
 * 
 * TO DO: Turn this class into an abstract superclass. Subclasses shall
 * implement the feedback component creation and specify where to locate the
 * feedback component relative to the underlying content component.
 * <p>
 * 
 * TO DO: Consider adding a mechanism, so that components can be added and
 * removed later.
 * 
 * @author Karsten Lentzsch
 * @author Damien Coraboeuf
 */
public final class IconFeedbackPanel extends JLayeredPane {

	private static final long serialVersionUID = 1L;

	private static final int CONTENT_LAYER = 1;

	private static final int FEEDBACK_LAYER = 2;

	private static final byte[] ERROR_GIF_BYTES = {71, 73, 70, 56, 57, 97, 7,
			0, 8, 0, -94, 0, 0, -1, 0, 0, -65, 63, 63, 127, 63, 63, -1, 127,
			127, -65, 95, 95, -33, -65, -65, -1, -1, -1, -1, -1, -1, 33, -7, 4,
			1, 0, 0, 7, 0, 44, 0, 0, 0, 0, 7, 0, 8, 0, 0, 3, 25, 88, -79, -85,
			54, 96, -104, 37, -55, 19, 1, 88, 2, -80, 54, 28, 86, 93, -63, 19,
			77, -118, -96, 6, 69, 2, 0, 59, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0};

	private static final ImageIcon ERROR_ICON = new ImageIcon(ERROR_GIF_BYTES);

	/**
	 * Wraps the components in the given component tree with instances of
	 * IconFeedbackPanel where necessary. Such a wrapper is required for all
	 * JScrollPanes that contain multiple children and for the root - unless
	 * it's a JScrollPane with multiple children.
	 * 
	 * @param model
	 *            Model for the validation results
	 * @param root
	 *            the root of the component tree to wrap
	 * @return the wrapped component tree
	 */
	public static JComponent getWrappedComponentTree(
			ValidationSupport<?> model, JComponent root) {
		wrapComponentTree(model, root);
		return isScrollPaneWithUnmarkableView(root)
				? root
				: new IconFeedbackPanel(model, root);
	}

	/**
	 * Returns the ValidationResult associated with the given component using
	 * the specified validation result key map. Unlike
	 * {@link ValidationComponentUtils#getAssociatedResult(JComponent, Map)}
	 * this method returns the empty result if the component has no keys set.
	 * 
	 * @param comp
	 *            the component may be marked with a validation message key
	 * @param keyMap
	 *            maps validation message keys to ValidationResults
	 * @return the ValidationResult associated with the given component as
	 *         provided by the specified validation key map or
	 *         <code>ValidationResult.EMPTY</code> if the component has no
	 *         message key set, or <code>ValidationResult.EMPTY</code> if no
	 *         result is associated with the component
	 */
	private static GUIValidation getAssociatedResult(JComponent comp,
			Map<Object, GUIValidation> keyMap) {
		GUIValidation result = ValidationComponentUtils.getAssociatedResult(
				comp, keyMap);
		return result == null ? GUIValidation.EMPTY : result;
	}

	// Instance Creation ******************************************************

	// Convenience Code *******************************************************

	/**
	 * Returns a string representation of the given validation result, intended
	 * for tool tips. Unlike {@link ValidationResult#getMessagesText()} this
	 * method returns an HTML string. It is invoked by
	 * {@link #createFeedbackComponent(ValidationResult, Component)} and the
	 * result won't be empty.
	 * 
	 * @param result
	 *            provides the ValidationMessages to iterate over
	 * @return an HTML representation of the given result
	 * 
	 * @since 1.3.1
	 */
	private static String getMessagesToolTipText(GUIValidation result) {
		StringBuilder builder = new StringBuilder("<html>");
		int index = 0;
		for (GUIValidationMessage message : result.getMessages()) {
			if (index > 0) {
				builder.append("<br>");
			}
			builder.append(message.formattedText());
			index++;
		}
		builder.append("</html>");
		return builder.toString();
	}

	/**
	 * Checks and answers if the given component can be marked or not.
	 * 
	 * 
	 * 
	 * @param component
	 *            the component to be checked
	 * @return true if the given component can be marked, false if not
	 */
	private static boolean isMarkable(Component component) {
		return component instanceof JTextComponent
				|| component instanceof JComboBox || component instanceof JFile
				|| component instanceof JSpinner;
	}

	private static boolean isScrollPaneView(Component c) {
		Container container = c.getParent();
		Container containerParent = container.getParent();
		return container instanceof JViewport
				&& containerParent instanceof JScrollPane;
	}

	// Initialization *********************************************************

	private static boolean isScrollPaneWithUnmarkableView(Component c) {
		if (!(c instanceof JScrollPane)) {
			return false;
		}
		JScrollPane scrollPane = (JScrollPane) c;
		JViewport viewport = scrollPane.getViewport();
		JComponent view = (JComponent) viewport.getView();
		return !isMarkable(view);
	}

	// Abstract Behavior ******************************************************

	private static void wrapComponentTree(ValidationSupport<?> model,
			Container container) {
		if (!(container instanceof JScrollPane)) {
			int componentCount = container.getComponentCount();
			for (int i = 0; i < componentCount; i++) {
				Component child = container.getComponent(i);
				if (child instanceof Container) {
					wrapComponentTree(model, (Container) child);
				}
			}
			return;
		}
		JScrollPane scrollPane = (JScrollPane) container;
		JViewport viewport = scrollPane.getViewport();
		JComponent view = (JComponent) viewport.getView();
		if (isMarkable(view)) {
			return;
		}
		// TO DO: Consider adding the following sanity check:
		// the view must not be an IconFeedbackPanel
		Component wrappedView = new IconFeedbackPanel(model, view);
		viewport.setView(wrappedView);
		wrapComponentTree(model, view);
	}

	/**
	 * Holds the ValidationResult and reports changes in that result. Used to
	 * update the state of the feedback components.
	 */
	private final ValidationSupport<?> validationSupport;

	/**
	 * Refers to the content panel that holds the content components.
	 */
	private final JComponent content;

	// Updating the Overlay Components ****************************************

	/**
	 * Creates an IconFeedbackPanel on the given ValidationResultModel using the
	 * specified content panel.
	 * <p>
	 * 
	 * <strong>Note:</strong> Typically you should wrap component trees with
	 * getWrappedComponentTree, not this constructor.
	 * <p>
	 * 
	 * <strong>Note:</strong> You must not add or remove components from the
	 * content once this constructor has been invoked.
	 * 
	 * @param validationSupport
	 *            the validation support
	 * @param content
	 *            the panel that contains the content components
	 * 
	 * @throws NullPointerException
	 *             if model or content is <code>null</code>.
	 */
	public IconFeedbackPanel(ValidationSupport<?> validationSupport,
			JComponent content) {
		if (validationSupport == null) {
			throw new NullPointerException(
					"The validation support must not be null.");
		}
		if (content == null) {
			throw new NullPointerException("The content must not be null.");
		}

		this.validationSupport = validationSupport;
		this.content = content;
		setLayout(new SimpleLayout());
		add(content, CONTENT_LAYER);
		initEventHandling();
	}

	/**
	 * Gets the icon for a validation result
	 * 
	 * @param result
	 *            Validation result
	 * @return Icon to use
	 */
	protected Icon getResultIcon(GUIValidation result) {
		Icon icon;
		if (result.hasErrors()) {
			icon = ERROR_ICON;
		} else {
			icon = null;
		}
		return icon;
	}

	/**
	 * Recursively descends the container tree and recomputes the layout for any
	 * subtrees marked as needing it (those marked as invalid). In addition to
	 * the superclass behavior, we reposition the feedback components after the
	 * child components have been validated.
	 * <p>
	 * 
	 * We reposition the feedback components only, if this panel is visible; if
	 * it becomes visible, #validateTree will be invoked.
	 * 
	 * @see Container#validateTree()
	 * @see #validate()
	 * @see #invalidate()
	 * @see #doLayout()
	 * @see Component#setVisible(boolean)
	 * @see LayoutManager
	 */
	@Override
	protected void validateTree() {
		super.validateTree();
		if (isVisible()) {
			repositionFeedbackComponents();
		}
	}

	private void addFeedbackComponent(Component contentComponent,
			JComponent messageComponent, Map<Object, GUIValidation> keyMap,
			int xOffset, int yOffset) {
		GUIValidation result = getAssociatedResult(messageComponent, keyMap);
		JComponent feedbackComponent = createFeedbackComponent(result,
				contentComponent);
		if (feedbackComponent == null) {
			return;
		}
		add(feedbackComponent, Integer.valueOf(FEEDBACK_LAYER));
		Point overlayPosition = getFeedbackComponentOrigin(feedbackComponent,
				contentComponent);
		overlayPosition.translate(xOffset, yOffset);
		feedbackComponent.setLocation(overlayPosition);
	}

	/**
	 * Creates and returns a validation feedback component that shall overlay
	 * the specified content component.
	 * <p>
	 * 
	 * This implementation returns a JLabel. The validation result's severity is
	 * used to lookup the label's icon; the result's message text is set as the
	 * label's tooltip text.
	 * <p>
	 * 
	 * TO DO: Turn this method into an abstract method if this class becomes an
	 * abstract superclass of general feedback overlay panels.
	 * 
	 * @param result
	 *            determines the label's icon and tooltip text
	 * @param contentComponent
	 *            the component to get overlaid feedback
	 * @return the feedback component that overlays the content component
	 * 
	 * @throws NullPointerException
	 *             if the result is <code>null</code>
	 */
	private JComponent createFeedbackComponent(GUIValidation result,
			Component contentComponent) {
		Icon icon = getResultIcon(result);
		JLabel label = new JLabel(icon);
		label.setToolTipText(getMessagesToolTipText(result));
		label.setSize(label.getPreferredSize());
		return label;
	}

	/**
	 * Computes and returns the origin of the given feedback component using the
	 * content component's origin.
	 * <p>
	 * 
	 * This implementation returns a JLabel. The validation result's severity is
	 * used to lookup the label's icon; the result's message text is set as the
	 * label's tooltip text.
	 * <p>
	 * 
	 * TO DO: Turn this method into an abstract method if this class becomes an
	 * abstract superclass of general feedback overlay panels.
	 * 
	 * @param feedbackComponent
	 *            the component that overlays the content
	 * @param contentComponent
	 *            the component to get overlaid feedback
	 * @return the feedback component's origin
	 * 
	 * @throws NullPointerException
	 *             if the feedback component or content component is
	 *             <code>null</code>
	 */
	private Point getFeedbackComponentOrigin(JComponent feedbackComponent,
			Component contentComponent) {
		boolean isLTR = contentComponent.getComponentOrientation()
				.isLeftToRight();
		int x = contentComponent.getX()
				+ (isLTR ? 0 : contentComponent.getWidth() - 1)
				- feedbackComponent.getWidth() / 2;
		int y = contentComponent.getY() + contentComponent.getHeight()
				- feedbackComponent.getHeight() + 2;

		return new Point(x, y);
	}

	/**
	 * Registers a listener with the validation result model that updates the
	 * feedback components.
	 */
	private void initEventHandling() {
		PropertyChangeSupport.subscribe(this.validationSupport
				.getSubscriberValidator(), this.validationSupport,
				"validation", new ValidationTask());
	}

	// Event Handling *********************************************************

	private void removeAllFeedbackComponents() {
		int componentCount = getComponentCount();
		for (int i = componentCount - 1; i >= 0; i--) {
			Component child = getComponent(i);
			int layer = getLayer(child);
			if (layer == FEEDBACK_LAYER) {
				remove(i);
			}
		}
	}

	/**
	 * Ensures that the feedback components are repositioned. Invoked by
	 * <code>#validate</code>, i. e. if this panel is laid out.
	 * <p>
	 * 
	 * TO DO: Improve this implementation to set only positions. The current
	 * implementation removes all components and re-adds them later.
	 */
	private void repositionFeedbackComponents() {
		updateFeedbackComponents();
	}

	private void updateFeedbackComponents() {
		removeAllFeedbackComponents();
		visitComponentTree(this.content, this.validationSupport.keyMap(), 0, 0);
		repaint();
	}

	/**
	 * Traverses the component tree starting at the given container and creates
	 * a feedback component for each JTextComponent that is associated with a
	 * message in the specified <code>keyMap</code>.
	 * <p>
	 * 
	 * The arguments passed to the feedback component creation method are the
	 * visited component and its associated validation sub result. This sub
	 * result is requested from the specified <code>keyMap</code> using the
	 * visited component's message key.
	 * 
	 * @param container
	 *            the component tree root
	 * @param keyMap
	 *            maps messages keys to associated validation results
	 */
	private void visitComponentTree(Container container,
			Map<Object, GUIValidation> keyMap, int xOffset, int yOffset) {
		int componentCount = container.getComponentCount();
		for (int i = 0; i < componentCount; i++) {
			Component child = container.getComponent(i);
			if (!child.isVisible()) {
				continue;
			}
			if (isMarkable(child)) {
				if (isScrollPaneView(child)) {
					Component containerParent = container.getParent();
					addFeedbackComponent(containerParent, (JComponent) child,
							keyMap, xOffset - containerParent.getX(), yOffset
									- containerParent.getY());
				} else {
					addFeedbackComponent(child, (JComponent) child, keyMap,
							xOffset, yOffset);
				}
			} else if (!isScrollPaneView(child) && child instanceof Container) {
				visitComponentTree((Container) child, keyMap, xOffset
						+ child.getX(), yOffset + child.getY());
			}
		}
	}

	// Layout *****************************************************************

	/**
	 * Used to lay out the content layer in the icon feedback JLayeredPane. The
	 * content fills the parent's space; minimum and preferred size of this
	 * layout are requested from the content panel.
	 */
	private final class SimpleLayout implements LayoutManager {

		/**
		 * If the layout manager uses a per-component string, adds the component
		 * <code>comp</code> to the layout, associating it with the string
		 * specified by <code>name</code>.
		 * 
		 * @param name
		 *            the string to be associated with the component
		 * @param comp
		 *            the component to be added
		 */
		public void addLayoutComponent(String name, Component comp) {
			// components are well known by the container
		}

		/**
		 * Lays out the specified container.
		 * 
		 * @param parent
		 *            the container to be laid out
		 */
		public void layoutContainer(Container parent) {
			Dimension size = parent.getSize();
			IconFeedbackPanel.this.content.setBounds(0, 0, size.width,
					size.height);
		}

		/**
		 * Calculates the minimum size dimensions for the specified container,
		 * given the components it contains.
		 * 
		 * @param parent
		 *            the component to be laid out
		 * @return the minimum size of the given container
		 * @see #preferredLayoutSize(Container)
		 */
		public Dimension minimumLayoutSize(Container parent) {
			return IconFeedbackPanel.this.content.getMinimumSize();
		}

		/**
		 * Calculates the preferred size dimensions for the specified container,
		 * given the components it contains.
		 * 
		 * @param parent
		 *            the container to be laid out
		 * @return the preferred size of the given container
		 * @see #minimumLayoutSize(Container)
		 */
		public Dimension preferredLayoutSize(Container parent) {
			return IconFeedbackPanel.this.content.getPreferredSize();
		}

		/**
		 * Removes the specified component from the layout.
		 * 
		 * @param comp
		 *            the component to be removed
		 */
		public void removeLayoutComponent(Component comp) {
			// components are well known by the container
		}

	}

	/**
	 * Gets notified when the ValidationResult changed and updates the feedback
	 * components.
	 */
	private final class ValidationTask implements Runnable {

		public void run() {
			updateFeedbackComponents();
		}

	}

}
