package net.sf.doolin.gui.validation.support;

import java.util.Collections;
import java.util.Map;

import net.sf.doolin.bus.bean.Bean;
import net.sf.doolin.bus.support.AndSubscriberValidator;
import net.sf.doolin.bus.support.SubscriberValidator;
import net.sf.doolin.bus.support.TriggeredSubscriberValidator;
import net.sf.doolin.gui.validation.GUIValidation;
import net.sf.doolin.gui.validation.GUIValidator;
import net.sf.doolin.gui.validation.ValidationSupport;
import net.sf.doolin.util.Utils;

/**
 * Implementation for the {@link ValidationSupport}.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of bean that is validated
 */
public class ValidationSupportImpl<B> extends Bean implements
		ValidationSupport<B> {

	private static final long serialVersionUID = 1L;

	private final B model;
	private final SubscriberValidator subscriberValidator;

	private GUIValidation validation;
	private GUIValidator<B> validator;

	private TriggeredSubscriberValidator triggerValidator;

	private boolean invalidation;

	/**
	 * Instantiates a new validation support.
	 * 
	 * @param model
	 *            the model that is validated
	 * @param subscriberValidator
	 *            the validator to use
	 */
	public ValidationSupportImpl(B model,
			SubscriberValidator subscriberValidator) {
		this.model = model;
		this.validation = new GUIValidation();
		this.subscriberValidator = subscriberValidator;
		// Initialization
		updateValidation();
	}

	@Override
	public SubscriberValidator getSubscriberValidator() {
		return this.subscriberValidator;
	}

	/**
	 * Gets the validation
	 * 
	 * @return the validation
	 */
	@Override
	public GUIValidation getValidation() {
		return this.validation;
	}

	@Override
	public void invalidate() {
		this.invalidation = true;
	}

	/**
	 * Checks the result of the validation
	 * 
	 * @return Validation result
	 */
	@Override
	public boolean isValid() {
		boolean valid = !this.invalidation
				&& (!isValidationEnabled() || this.validation == null || !this.validation
						.hasErrors());
		this.invalidation = false;
		return valid;
	}

	/**
	 * Checks if is validation enabled.
	 * 
	 * @return true, if is validation enabled
	 */
	@Override
	public boolean isValidationEnabled() {
		return this.validator != null;
	}

	@Override
	public Map<Object, GUIValidation> keyMap() {
		if (this.validation != null) {
			return this.validation.keyMap();
		} else {
			return Collections.emptyMap();
		}
	}

	/**
	 * Sets the validator to use
	 * 
	 * @param newValidator
	 *            Validator to use
	 */
	@Override
	public void setValidator(GUIValidator<B> newValidator) {
		if (this.validator != null) {
			this.triggerValidator.stop();
		}
		this.validator = newValidator;
		if (this.validator != null) {
			this.triggerValidator = new TriggeredSubscriberValidator();
			AndSubscriberValidator currentValidator = new AndSubscriberValidator(
					this.triggerValidator, this.subscriberValidator);
			this.validator.subscribe(currentValidator, this.model,
					new Runnable() {

						@Override
						public void run() {
							updateValidation();
						}
					});
		}
		if (this.subscriberValidator.isValid()) {
			updateValidation();
		}
	}

	@Override
	public String toString() {
		String value = Utils.simpleToString(this.model, "<null>");
		return "ValidationSupportImpl/" + value;
	}

	/**
	 * Updates the validation state.
	 * 
	 * @see GUIValidator#validate(Object)
	 */
	protected void updateValidation() {
		if (this.validator != null) {
			GUIValidation old = this.validation;
			this.validation = this.validator.validate(this.model);
			notify("validation", old, this.validation);
		}
	}

}
