package net.sf.doolin.gui.view.descriptor;

import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.action.path.ActionBarContext;
import net.sf.doolin.gui.action.path.list.ActionPathList;
import net.sf.doolin.gui.action.path.toolbar.ToolBar;
import net.sf.doolin.gui.action.swing.ActionFactory;
import net.sf.doolin.gui.action.swing.DefaultActionFactory;
import net.sf.doolin.gui.expression.GUIExpression;
import net.sf.doolin.gui.validation.GUIValidator;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.view.decorator.GUIViewDecorator;
import net.sf.doolin.gui.view.support.GUIViewAdapter;
import net.sf.doolin.gui.view.support.ViewSubscriber;
import net.sf.doolin.gui.window.GUIWindow;
import net.sf.jstring.LocalizableException;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.LinkedList;
import java.util.List;

/**
 * Abstract view descriptor.
 * 
 * @param <V>
 *            Type of object for the view
 * 
 * @author Damien Coraboeuf
 */
public abstract class AbstractGUIViewDescriptor<V> implements
		GUIViewDescriptor<V>, BeanNameAware {

	private String viewId;

	private String titleExpression = "";

	private String iconId;

	private GUIValidator<V> validator;

	private GUIViewDecorator viewDecorator;

	private GUIAction closeAction;

	private ActionFactory actionFactory = DefaultActionFactory.INSTANCE;

	private ActionPathList windowMenuBar;
	private ToolBar windowToolBar;
	private ActionPathList viewToolBar;

	private String parentViewId;

	private List<ViewSubscriber<V>> viewSubscribers;

	// Opened views for this descriptor
	private LinkedList<GUIView<V>> views = new LinkedList<GUIView<V>>();

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean close(GUIView<V> view, boolean force) {
		// Executes the close action
		if (this.closeAction != null) {
			this.closeAction.execute(view.getActionContext());
			if (view.isClosed()) {
				return true;
			} else if (force) {
				view.close();
				return view.isClosed();
			} else {
				return false;
			}
		} else {
			view.close();
			return view.isClosed();
		}
	}

	public ActionFactory getActionFactory() {
		return this.actionFactory;
	}

	/**
	 * Gets the close action.
	 * 
	 * @return the close action
	 */
	public GUIAction getCloseAction() {
		return this.closeAction;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getIconId() {
		return this.iconId;
	}

	/**
	 * {@inheritDoc}.
	 * 
	 * @param model
	 *            the model
	 * 
	 * @return the ID
	 */
	@Override
	public String getID(V model) {
		GUIExpression expression = new GUIExpression(null, model, this.viewId);
		return expression.getValue();
	}

	/**
	 * Gets the ID of any parent view for this view.
	 * 
	 * @return ID of the parent view
	 */
	public String getParentViewId() {
		return this.parentViewId;
	}

	/**
	 * Gets the title expression.
	 * 
	 * @return the title expression
	 */
	public String getTitleExpression() {
		return this.titleExpression;
	}

	/**
	 * Gets the validator.
	 * 
	 * @return the validator
	 */
	public GUIValidator<V> getValidator() {
		return this.validator;
	}

	/**
	 * {@inheritDoc}.
	 * 
	 * @return the view decorator
	 */
	public GUIViewDecorator getViewDecorator() {
		return this.viewDecorator;
	}

	/**
	 * Gets the view id.
	 * 
	 * @return the view id
	 */
	@Override
	public String getViewId() {
		return this.viewId;
	}

	/**
	 * Gets the list of {@link ViewSubscriber} to execute for the view
	 * 
	 * @return List of {@link ViewSubscriber}
	 */
	public List<ViewSubscriber<V>> getViewSubscribers() {
		return this.viewSubscribers;
	}

	/**
	 * Gets the tool bar definition for this view
	 * 
	 * @return Tool bar definition
	 */
	public ActionPathList getViewToolBar() {
		return this.viewToolBar;
	}

	/**
	 * Gets the definition of any view-specific menu bar to merge with the
	 * window's menu bar.
	 * 
	 * @return Menu bar definition
	 */
	public ActionPathList getWindowMenuBar() {
		return this.windowMenuBar;
	}

	/**
	 * Gets the definition of any view-specific tool bar to merge with the
	 * window tool bars.
	 * 
	 * @return Tool bar definition
	 */
	public ToolBar getWindowToolBar() {
		return this.windowToolBar;
	}

	/**
	 * General initialisation of the view.
	 * 
	 * @param view
	 *            View to initialise
	 */
	protected void initView(GUIView<V> view) {
		// View tool bar
		if (this.viewToolBar != null) {
			view.installToolBar(this.viewToolBar, view.getActionContext());
		}
		// Listens to view events
		view.addViewListener(new GUIViewAdapter<V>() {

			@Override
			public void onViewClosed(GUIView<V> view) {
				// Removes the view from opened views
				AbstractGUIViewDescriptor.this.views.remove(view);
				// No more view
				if (AbstractGUIViewDescriptor.this.views.isEmpty()) {
					// Uninstalls the tool bar with the descriptor id
					if (AbstractGUIViewDescriptor.this.windowToolBar != null) {
						view.getParentWindow().uninstallToolBar(
								AbstractGUIViewDescriptor.this.windowToolBar
										.getName());
					}
					// Uninstalls menu items
					view.getParentWindow().uninstallMenuBar(
							AbstractGUIViewDescriptor.this.viewId);
				}
				// Activates the last view
				else {
					onActivateView(AbstractGUIViewDescriptor.this.views
							.getLast());
				}
			}

			@Override
			public void onViewInit(GUIView<V> view) {
				// Adds the view to the opened views
				AbstractGUIViewDescriptor.this.views.remove(view);
				AbstractGUIViewDescriptor.this.views.add(view);
			}

			@Override
			public void onViewShown(GUIView<V> view) {
				// Activation of the tool bar
				onActivateView(view);
			}

		});
		// Parent view
		if (StringUtils.isNotBlank(this.parentViewId)) {
			// Gets the actual view id
			String actualParentViewId = new GUIExpression(null, view
					.getViewData(), this.parentViewId).getValue();
			// Gets the parent view
			GUIView<?> parentView = view.getParentWindow().getView(
					actualParentViewId);
			if (parentView == null) {
				throw new LocalizableException("GUIViewDescriptor.NoParentView",
						actualParentViewId);
			}
			// Listens for the closing of the parent
			registerParent(view, parentView);
		}
		// View subscriptions
		if (this.viewSubscribers != null) {
			for (ViewSubscriber<V> viewSubscriber : this.viewSubscribers) {
				viewSubscriber.subscribe(view);
			}
		}
	}

	/**
	 * Activates the view and its tool bars.
	 * 
	 * @param view
	 *            View
	 */
	protected void onActivateView(GUIView<V> view) {
		// Window tool bar
		if (this.windowToolBar != null) {
			GUIWindow<?> parentWindow = view.getParentWindow();
			parentWindow.installToolBar(this.windowToolBar, view
					.getActionContext());
		}
		// Window menu bar
		if (this.windowMenuBar != null) {
			ActionBarContext menuBar = new ActionBarContext(this.windowMenuBar,
					view.getActionContext());
			GUIWindow<?> parentWindow = view.getParentWindow();
			parentWindow.installMenuBar(this.viewId, menuBar);
		}
	}

	/**
	 * Notifies the parent view that a view has been added to it
	 * 
	 * @param <P>
	 *            Type of bean for the parent view
	 * @param view
	 *            Child view
	 * @param parentView
	 *            Parent view to register the child view into
	 */
	protected <P> void registerParent(final GUIView<V> view,
			GUIView<P> parentView) {
		parentView.addViewListener(new GUIViewAdapter<P>() {
			@Override
			public void onViewClosed(GUIView<P> v) {
				view.close();
			}
		});
	}

	/**
	 * Sets the action factory.
	 * 
	 * @param actionFactory
	 *            the new action factory
	 */
	@Autowired(required = false)
	public void setActionFactory(ActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setBeanName(String name) {
		if (StringUtils.isBlank(this.viewId)) {
			this.viewId = name;
		}
	}

	/**
	 * Sets the close action.
	 * 
	 * @param closeAction
	 *            the new close action
	 */
	public void setCloseAction(GUIAction closeAction) {
		this.closeAction = closeAction;
	}

	/**
	 * Sets the icon id.
	 * 
	 * @param iconId
	 *            the new icon id
	 */
	public void setIconId(String iconId) {
		this.iconId = iconId;
	}

	/**
	 * Sets the expression that designates the ID of the parent view
	 * 
	 * @param parentViewId
	 *            Expression that designates the ID of the parent view
	 */
	public void setParentViewId(String parentViewId) {
		this.parentViewId = parentViewId;
	}

	/**
	 * Sets the title expression.
	 * 
	 * @param titleExpression
	 *            the new title expression
	 */
	public void setTitleExpression(String titleExpression) {
		this.titleExpression = titleExpression;
	}

	/**
	 * Sets the validator.
	 * 
	 * @param validator
	 *            the new validator
	 */
	public void setValidator(GUIValidator<V> validator) {
		this.validator = validator;
	}

	/**
	 * Sets the view decorator.
	 * 
	 * @param viewDecorator
	 *            the new view decorator
	 */
	public void setViewDecorator(GUIViewDecorator viewDecorator) {
		this.viewDecorator = viewDecorator;
	}

	/**
	 * Sets the expression for the view id.
	 * 
	 * @param viewId
	 *            the new view id
	 */
	public void setViewId(String viewId) {
		this.viewId = viewId;
	}

	/**
	 * Sets the list of {@link ViewSubscriber} to execute for the view
	 * 
	 * @param viewSubscribers
	 *            List of {@link ViewSubscriber}
	 */
	public void setViewSubscribers(List<ViewSubscriber<V>> viewSubscribers) {
		this.viewSubscribers = viewSubscribers;
	}

	/**
	 * Sets the tool bar definition for this view
	 * 
	 * @param viewToolBar
	 *            Tool bar definition or <code>null</code> if none is needed
	 */
	public void setViewToolBar(ActionPathList viewToolBar) {
		this.viewToolBar = viewToolBar;
	}

	/**
	 * Sets the definition of any view-specific menu bar to merge with the
	 * window's menu bar.
	 * 
	 * @param windowMenuBar
	 *            Menu bar definition or <code>null</code> if none is needed
	 */
	public void setWindowMenuBar(ActionPathList windowMenuBar) {
		this.windowMenuBar = windowMenuBar;
	}

	/**
	 * Sets the definition of any view-specific tool bar to merge with the
	 * window tool bars.
	 * 
	 * @param windowToolBar
	 *            Tool bar definition or <code>null</code> if none is needed
	 */
	public void setWindowToolBar(ToolBar windowToolBar) {
		this.windowToolBar = windowToolBar;
	}

}
