package net.sf.doolin.gui.view.support;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.app.GUIApplication;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.view.GUIViewListener;
import net.sf.doolin.gui.view.descriptor.GUIViewDescriptor;
import net.sf.doolin.gui.window.GUIWindow;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Abstract view
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 */
public abstract class AbstractGUIView<V> implements GUIView<V> {

	private static final Logger log = LoggerFactory
			.getLogger(AbstractGUIView.class);

	private final GUIWindow<?> parentWindow;
	private final GUIViewDescriptor<V> viewDescriptor;
	private final V model;
	private final Map<Object, Object> properties;

	private boolean closed;
	private List<GUIViewListener<V>> viewListeners;

	private final ViewActionContext<V> actionContext;

	/**
	 * Constructor.
	 * 
	 * @param parentWindow
	 *            Parent window
	 * @param model
	 *            View model
	 * @param viewDescriptor
	 *            the view descriptor
	 */
	public AbstractGUIView(GUIWindow<?> parentWindow,
			GUIViewDescriptor<V> viewDescriptor, V model) {
		this.parentWindow = parentWindow;
		this.viewDescriptor = viewDescriptor;
		this.model = model;
		this.properties = new HashMap<Object, Object>(1);
		this.closed = false;
		this.actionContext = new ViewActionContext<V>(this);
	}

	@Override
	public synchronized void addViewListener(GUIViewListener<V> listener) {
		if (this.viewListeners == null) {
			this.viewListeners = new ArrayList<GUIViewListener<V>>();
		}
		this.viewListeners.add(listener);
	}

	@Override
	public void close() {
		getParentWindow().removeView(this);
		this.closed = true;
		fireClosed();
	}

	/**
	 * Delegates to the view descriptor.
	 * 
	 * {@inheritDoc}
	 */
	@Override
	public boolean close(boolean force) {
		return this.viewDescriptor.close(this, force);
	}

	/**
	 * Fires that this view is closed.
	 * 
	 * @see GUIViewListener#onViewClosed(GUIView)
	 */
	protected void fireClosed() {
		Collection<GUIViewListener<V>> tViewListeners = getViewListenerArray();
		if (tViewListeners != null) {
			for (GUIViewListener<V> viewListener : tViewListeners) {
				viewListener.onViewClosed(this);
			}
		}
	}

	/**
	 * Fires that this view is initialised.
	 * 
	 * @see GUIViewListener#onViewInit(GUIView)
	 */
	protected void fireInit() {
		Collection<GUIViewListener<V>> tViewListeners = getViewListenerArray();
		if (tViewListeners != null) {
			for (GUIViewListener<V> viewListener : tViewListeners) {
				viewListener.onViewInit(this);
			}
		}
	}

	/**
	 * Fires that this view is shown.
	 * 
	 * @see GUIViewListener#onViewShown(GUIView)
	 */
	protected void fireShown() {
		Collection<GUIViewListener<V>> tViewListeners = getViewListenerArray();
		if (tViewListeners != null) {
			for (GUIViewListener<V> viewListener : tViewListeners) {
				viewListener.onViewShown(this);
			}
		}
	}

	@Override
	public ActionContext getActionContext() {
		return this.actionContext;
	}

	/**
	 * Gets the application.
	 * 
	 * @return the application
	 */
	public GUIApplication getApplication() {
		return this.parentWindow.getApplication();
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see GUIViewDescriptor#getID(Object)
	 */
	@Override
	public String getID() {
		return getViewDescriptor().getID(this.model);
	}

	/**
	 * Gets the parent window.
	 * 
	 * @return the parent window
	 */
	public GUIWindow<?> getParentWindow() {
		return this.parentWindow;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Object getProperty(Object key) {
		return this.properties.get(key);
	}

	/**
	 * Gets the model.
	 * 
	 * @return the model
	 */
	@Override
	public V getViewData() {
		return this.model;
	}

	/**
	 * {@inheritDoc}.
	 * 
	 * @return the view descriptor
	 */
	@Override
	public GUIViewDescriptor<V> getViewDescriptor() {
		return this.viewDescriptor;
	}

	/**
	 * Gets the list of view listeners
	 * 
	 * @return View listeners
	 */
	protected synchronized Collection<GUIViewListener<V>> getViewListenerArray() {
		if (this.viewListeners != null) {
			return new ArrayList<GUIViewListener<V>>(this.viewListeners);
		} else {
			return null;
		}
	}

	@Override
	public void init() {
		if (log.isDebugEnabled()) {
			log.debug(String.format("Created view is %s", getID()));
		}
		this.closed = false;
		fireInit();
	}

	@Override
	public boolean isClosed() {
		return this.closed;
	}

	@Override
	public synchronized void removeViewListener(GUIViewListener<V> listener) {
		if (this.viewListeners != null) {
			this.viewListeners.remove(listener);
			if (this.viewListeners.isEmpty()) {
				this.viewListeners = null;
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setProperty(Object key, Object value) {
		if (value != null) {
			this.properties.put(key, value);
		} else {
			this.properties.remove(key);
		}
	}

	@Override
	public void show() {
		if (log.isDebugEnabled()) {
			log.debug(String.format("Activated view is %s", getID()));
		}
		getParentWindow().showView(this);
		fireShown();
	}

}
