package net.sf.doolin.gui.window;

import java.util.Collection;

import javax.swing.Icon;
import javax.swing.JPanel;

import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.action.path.ActionBarContext;
import net.sf.doolin.gui.action.path.toolbar.ToolBar;
import net.sf.doolin.gui.app.GUIApplication;
import net.sf.doolin.gui.util.PropertyContainer;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.window.descriptor.GUIWindowDescriptor;

/**
 * This interface defines a main application window.
 * 
 * @param <B>
 *            Type of data for the window
 */
public interface GUIWindow<B> extends PropertyContainer {

	/**
	 * Adds a listener for the window
	 * 
	 * @param listener
	 *            Window listener
	 */
	void addGUIWindowListener(GUIWindowListener<B> listener);

	/**
	 * Registers a new view in this window
	 * 
	 * @param view
	 *            View to register
	 */
	void addView(GUIView<?> view);

	/**
	 * Builds and initializes the window.
	 * 
	 * @param application
	 *            Owning application
	 * @param windowDescriptor
	 *            The window descriptor
	 * @param windowData
	 *            Associated data
	 */
	void build(GUIApplication application,
			GUIWindowDescriptor<B> windowDescriptor, B windowData);

	/**
	 * Checks if the window can be closed
	 * 
	 * @return <code>true</code> if the window can be closed
	 */
	boolean canClose();

	/**
	 * Closes the window.
	 */
	void close();

	/**
	 * Displays the window on the screen.
	 */
	void display();

	/**
	 * Gets the action context for this window
	 * 
	 * @return Action context
	 */
	ActionContext getActionContext();

	/**
	 * Gets the application.
	 * 
	 * @return the application
	 */
	GUIApplication getApplication();

	/**
	 * Gets the ID for this window
	 * 
	 * @return ID
	 */
	String getId();

	/**
	 * List of open views
	 * 
	 * @return List of open views
	 */
	Collection<? extends GUIView<?>> getOpenViews();

	/**
	 * Gets an already opened view
	 * 
	 * @param viewId
	 *            View ID
	 * @return Already opened view or <code>null</code>.
	 * @param <V>
	 *            Type of data for the view
	 */
	<V> GUIView<V> getView(String viewId);

	/**
	 * Returns the window data
	 * 
	 * @return Window data
	 */
	B getWindowData();

	/**
	 * Gets the window descriptor.
	 * 
	 * @return the window descriptor
	 */
	GUIWindowDescriptor<B> getWindowDescriptor();

	/**
	 * Content panel.
	 * 
	 * @return Content panel for this window
	 */
	JPanel getWindowPanel();

	/**
	 * Hides a view
	 * 
	 * @param view
	 *            View to hide
	 */
	void removeView(GUIView<?> view);

	/**
	 * Installs a menu bar in the window for a given context (which can be
	 * different form the window's one)
	 * 
	 * @param id
	 *            ID of the menu bar
	 * @param menuBar
	 *            Action bar context
	 */
	void installMenuBar(String id, ActionBarContext menuBar);

	/**
	 * Installs a tool bar in the window
	 * 
	 * @param toolBar
	 *            Tool bar to install
	 * @param actionContext
	 *            Action context
	 */
	void installToolBar(ToolBar toolBar, ActionContext actionContext);

	/**
	 * Checks if the window is closed
	 * 
	 * @return <code>true</code> if the window is closed
	 */
	boolean isClosed();

	/**
	 * This method is called when the window has been fully created
	 */
	void onPostCreate();

	/**
	 * Sets the icon for the window
	 * 
	 * @param icon
	 *            Icon to set
	 */
	void setWindowIcon(Icon icon);

	/**
	 * Sets the window title.
	 * 
	 * @param value
	 *            Window title
	 */
	void setWindowTitle(String value);

	/**
	 * Activates a view
	 * 
	 * @param view
	 *            View to activate
	 */
	void showView(GUIView<?> view);

	/**
	 * Uninstalls a menu bar with the given id
	 * 
	 * @param id
	 *            ID of the menu bar to uninstall
	 */
	void uninstallMenuBar(String id);

	/**
	 * Uninstalls a tool bar with the given id
	 * 
	 * @param id
	 *            ID of the tool bar to uninstall
	 */
	void uninstallToolBar(String id);

}
