package net.sf.doolin.gui.window.dialog;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;

import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.action.path.ActionBarContext;
import net.sf.doolin.gui.action.path.item.ActionPath;
import net.sf.doolin.gui.action.path.item.GUIActionPath;
import net.sf.doolin.gui.action.path.list.DefaultActionPathList;
import net.sf.doolin.gui.service.GUIPreferences;
import net.sf.doolin.gui.swing.JButtonBar;
import net.sf.doolin.gui.window.support.AbstractRootPaneGUIWindow;
import net.sf.doolin.gui.window.support.IWindow;
import net.sf.doolin.gui.window.support.IWindowFactory;
import net.sf.doolin.gui.window.support.WindowSize;
import net.sf.doolin.util.collection.ListUtils;

/**
 * Window for a dialog. It can rely on a {@link JDialog} for modal wizards or on
 * a {@link JFrame} for modeless wizards.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of data hold by the window
 * @see IWindow
 */
public abstract class AbstractDialogWindow<B> extends
		AbstractRootPaneGUIWindow<B> {

	private final DialogActionsFactory dialogActionsFactory;

	private IWindow dialog;
	private JPanel dialogPanel;
	private JPanel commandPanel;
	private JButtonBar buttonBar;
	private WindowSize size;

	private Map<String, GUIAction> dialogActionMap;

	/**
	 * Constructor
	 * 
	 * @param dialogActionsFactory
	 *            Factory for the dialog actions (ok, cancel, previous, next,
	 *            finish, cancel...)
	 */
	public AbstractDialogWindow(DialogActionsFactory dialogActionsFactory) {
		this.dialogActionsFactory = dialogActionsFactory;
	}

	protected void addWindowListener(WindowListener listener) {
		this.dialog.addWindowListener(listener);
	}

	@Override
	protected void build() {
		// Default
		super.build();
		// Gets dialog information
		IWindowFactory iwindowFactory = getIwindowFactory();
		boolean modal = isModal();
		// Creates the dialog
		this.dialog = iwindowFactory.createIWindow(getActionContext(), modal);
		// Content pane
		Container contentPane = this.dialog.getContentPane();
		contentPane.setLayout(new BorderLayout());
		// Dialog panel
		this.dialogPanel = new JPanel(new BorderLayout());
		contentPane.add(this.dialogPanel, BorderLayout.CENTER);
		// Window panel
		this.dialogPanel.add(getRootPanel(), BorderLayout.CENTER);
		// Command bar
		this.commandPanel = new JPanel(new BorderLayout());
		this.commandPanel.setBorder(BorderFactory.createRaisedBevelBorder());
		this.dialogPanel.add(this.commandPanel, BorderLayout.PAGE_END);
	}

	/**
	 * Closes the dialog
	 */
	@Override
	public void close() {
		if (this.size != null && this.size.isSaveInPrefs()) {
			int width = this.dialog.getWidth();
			int height = this.dialog.getHeight();
			// Preferences key
			String prefKey = getId() + ".size";
			// Sets the preferences
			GUIPreferences preferences = getApplication().getPreferences();
			// Sets the preferences
			preferences.setInt(prefKey + ".width", width);
			preferences.setInt(prefKey + ".height", height);
			// Saves the preferences
			preferences.save();
		}
		// Closes the dialog
		this.dialog.dispose();
		// Notifies the window descriptor
		super.close();
	}

	@Override
	public void display() {
		// Checks
		if (this.dialog == null) {
			throw new IllegalStateException("The dialog has not been created.");
		}
		// First display
		this.dialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowOpened(WindowEvent e) {
				getWindowDescriptor().onOpenedWindow(AbstractDialogWindow.this);
			}
		});
		// Size
		setSize(getWindowDescriptor().getSize());
		// Shows the dialog
		this.dialog.show();
	}

	/**
	 * Returns the paths for the dialog actions
	 * 
	 * @return Paths for the actions
	 * @see DialogActionsFactory#getDialogActions()
	 */
	protected ActionBarContext getActionBarContext() {
		// List of dialog actions
		List<GUIAction> dialogActionList = this.dialogActionsFactory
				.getDialogActions();

		// Indexation of actions using their ID
		this.dialogActionMap = ListUtils.indexMap(dialogActionList, "id");

		// paths
		DefaultActionPathList actionPaths = new DefaultActionPathList();
		List<ActionPath> paths = new ArrayList<ActionPath>();
		for (GUIAction dialogAction : dialogActionList) {
			paths.add(new GUIActionPath(dialogAction));
		}
		actionPaths.setPaths(paths);
		// OK
		return new ActionBarContext(actionPaths, getActionContext());
	}

	protected JButton getButton(String name) {
		if (this.buttonBar != null) {
			return this.buttonBar.getButton(name);
		} else {
			return null;
		}
	}

	/**
	 * Gets the close action for this dialog. By default, it returns the
	 * "Cancel" action, then the "cancelAction" of the window descriptor, then
	 * null.
	 */
	protected GUIAction getCloseAction() {
		GUIAction cancelAction = this.dialogActionMap.get(DialogAction.CANCEL);
		if (cancelAction != null) {
			return cancelAction;
		} else {
			return getWindowDescriptor().getCloseAction();
		}
	}

	protected abstract IWindowFactory getIwindowFactory();

	protected abstract boolean isModal();

	/**
	 * Creates the button bar
	 */
	@Override
	public void onPostCreate() {
		super.onPostCreate();
		// Creates the button bar
		ActionBarContext actionBarContext = getActionBarContext();
		if (actionBarContext.isEmpty()) {
			// No action is defined
			// Hides the command bar
			this.commandPanel.setVisible(false);
			// OK
			return;
		}
		this.buttonBar = getWindowDescriptor().getActionFactory()
				.createButtonBar(actionBarContext);
		// Adds the button bar in the command panel
		this.commandPanel.add(this.buttonBar, BorderLayout.CENTER);

		// Window close action
		this.dialog.addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent e) {
				GUIAction action = getCloseAction();
				if (action != null) {
					action.execute(getActionContext());
				}
			}

		});
	}

	protected void setDefaultButton(JButton button) {
		this.dialog.setDefaultButton(button);
	}

	/**
	 * Sets the size of the dialog
	 * 
	 * @param size
	 *            Size specification
	 */
	protected void setSize(WindowSize size) {
		this.dialog.getContentPane().doLayout();
		this.size = size;
		int width = size.getWidth();
		int height = size.getHeight();
		boolean pack = size.isPack();
		// Simple size
		this.dialog.setSize(width, height);
		// Location
		this.dialog.setLocation();
		// Gets the size parameters from preferences
		if (size.isSaveInPrefs()) {
			// Preferences key
			String prefKey = getId() + ".size";
			// Gets the preferences
			GUIPreferences preferences = getApplication().getPreferences();
			// Reads the preferences
			width = preferences.getInt(prefKey + ".width", width);
			height = preferences.getInt(prefKey + ".height", height);
		}
		// Pack
		if (pack) {
			this.dialog.pack();
		}
		// Simple size
		else {
			this.dialog.setSize(width, height);
		}
		// Location
		this.dialog.setLocation();
	}

	@Override
	public void setWindowIcon(Icon icon) {
		this.dialog.setIcon(icon);
	}

	@Override
	public void setWindowTitle(String value) {
		this.dialog.setTitle(value);
	}

}
