package net.sf.doolin.gui.window.support;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Icon;
import javax.swing.JFrame;

import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.service.GUIPreferences;
import net.sf.doolin.gui.swing.SwingUtils;

/**
 * Application window based on a {@link JFrame}.
 * 
 * @param <B>
 *            Type of the window data
 * 
 * @author Damien Coraboeuf
 */
public class FrameGUIWindow<B> extends AbstractRootPaneGUIWindow<B> {

	private JFrame frame;
	private WindowSize size;

	/**
	 * Saves in preferences when the frame is closed
	 * 
	 * {@inheritDoc}
	 */
	@Override
	public void close() {
		if (this.size != null && this.size.isSaveInPrefs()) {
			int width = this.frame.getWidth();
			int height = this.frame.getHeight();
			boolean maximized = (this.frame.getExtendedState() == Frame.MAXIMIZED_BOTH);
			// Preferences key
			String prefKey = getId() + ".size";
			// Sets the preferences
			GUIPreferences preferences = getApplication().getPreferences();
			// Sets the preferences
			preferences.setInt(prefKey + ".width", width);
			preferences.setInt(prefKey + ".height", height);
			preferences.setBoolean(prefKey + ".maximized", maximized);
			// Saves the preferences
			preferences.save();
		}
		// Closes the frame
		this.frame.dispose();
		// Notifies the window descriptor
		super.close();
	}

	/**
	 * {@inheritDoc}.
	 */
	@Override
	public void display() {
		// Checks
		if (this.frame == null) {
			throw new IllegalStateException("The frame has not been created.");
		}
		// First display
		this.frame.addWindowListener(new WindowAdapter() {
			@Override
			public void windowOpened(WindowEvent e) {
				getWindowDescriptor().onOpenedWindow(FrameGUIWindow.this);
			}
		});
		// Size
		setSize(getWindowDescriptor().getSize());
		// Checks if the frame is visible
		if (this.frame.isVisible()) {
			// Brings it to the front
			this.frame.toFront();
		}
		// Shows the frame
		else {
			this.frame.setVisible(true);
			this.frame.toFront();
		}
	}

	@Override
	public void setWindowIcon(Icon icon) {
		this.frame.setIconImage(SwingUtils.toImage(icon));
	}

	/**
	 * Sets the frame title.
	 * 
	 * {@inheritDoc}
	 */
	@Override
	public void setWindowTitle(String value) {
		this.frame.setTitle(value);
	}

	/**
	 * {@inheritDoc}.
	 */
	@Override
	protected void build() {
		super.build();
		// Creates the frame
		this.frame = new JFrame();
		// Frame closing action
		this.frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		this.frame.addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent e) {
				GUIAction action = getWindowDescriptor().getCloseAction();
				if (action != null) {
					action.execute(getActionContext());
				}
			}

		});
		// Content pane
		Container contentPane = this.frame.getContentPane();
		contentPane.setLayout(new BorderLayout());
		// Window panel
		contentPane.add(getRootPanel(), BorderLayout.CENTER);
	}

	/**
	 * Sets the size of the frame
	 * 
	 * @param size
	 *            Size specification
	 */
	protected void setSize(WindowSize size) {
		if (this.frame.isVisible()) {
			return;
		}
		this.size = size;
		boolean pack = size.isPack();
		boolean maximized = size.isMaximized();
		int width = size.getWidth();
		int height = size.getHeight();
		// Gets the size parameters from preferences
		if (size.isSaveInPrefs()) {
			// Preferences key
			String prefKey = getId() + ".size";
			// Gets the preferences
			GUIPreferences preferences = getApplication().getPreferences();
			// Reads the preferences
			width = preferences.getInt(prefKey + ".width", width);
			height = preferences.getInt(prefKey + ".height", height);
			maximized = preferences.getBoolean(prefKey + ".maximized",
					maximized);
		}
		// Pack
		if (pack) {
			this.frame.pack();
		}
		// Simple size
		else {
			this.frame.setSize(width, height);
		}
		// Zoom
		if (maximized) {
			this.frame.setExtendedState(Frame.MAXIMIZED_BOTH);
		}
		// Not zoomed
		else if (this.size.isCentered()) {
			this.frame.setLocationRelativeTo(null);
		} else {
			this.frame.setLocationByPlatform(true);
		}
	}

}
